#pragma once 
#include <ArduinoBLE.h>
#include <map>

//#warning infinite loop in discoverDescriptors is disabled in ATT.cpp line 1548 of ArduinoBLE driver
// discover descriptors
//if (!discoverDescriptors(connHandle, device)) {
//  return false;
//}
  
class issBLE {
  std::map<String, BLEDevice> BLEdevices;
  BLEDevice currentDevice;
  
  public:

  BLEDevice getCurrentDevice() {
    return currentDevice;
  }
  
  void init() {
    // begin initialization
    if (!BLE.begin()) {
      DEBUG_BLE_PRINT("starting BLE failed!");  
      while (1);
    }  
    DEBUG_BLE_PRINT("BLE Central - Device Scanner");
  }

  void end() {
    BLE.end();
  }

  bool findDevice(String macaddress, unsigned msLimit = 2000) {
  auto startMs = millis();
  BLE.scan();
  
  while (millis() - startMs < msLimit) {
    currentDevice = BLE.available();
    if (currentDevice) {
      // discovered a peripheral, print out address, local name, and advertised service
      DEBUG_BLE_PRINT("Found " +  String(currentDevice.address()) + " " + String(currentDevice.localName()) + " " + 
        String(currentDevice.advertisedServiceUuid()));

      if (currentDevice.address() == macaddress) {
          BLE.stopScan();
          return true;
      }
    }
  }
  BLE.stopScan();
  return false;
}

void explorerPeripheral(BLEDevice peripheral) {
  // connect to the peripheral
  DEBUG_BLE_PRINT("Connecting ...");

  if (peripheral.connect()) {
    DEBUG_BLE_PRINT("Connected");
  } else {
    DEBUG_BLE_PRINT("Failed to connect!");
    return;
  }

  // discover peripheral attributes
  DEBUG_BLE_PRINT("Discovering attributes ...");
  if (peripheral.discoverAttributes()) {
    DEBUG_BLE_PRINT("Attributes discovered");
  } else {
    DEBUG_BLE_PRINT("Attribute discovery failed!");
    peripheral.disconnect();
    return;
  }

  // read and print device name of peripheral
  DEBUG_BLE_PRINT("Device name: " + peripheral.deviceName() + " Appearance: 0x" + peripheral.appearance());

  // loop the services of the peripheral and explore each
  for (int i = 0; i < peripheral.serviceCount(); i++) {
    BLEService service = peripheral.service(i);

    exploreService(service);
  }

  DEBUG_BLE_PRINT("");

  // we are done exploring, disconnect
  DEBUG_BLE_PRINT("Disconnecting ...");
  peripheral.disconnect();
  DEBUG_BLE_PRINT("Disconnected");
}

void exploreService(BLEService service) {
  // print the UUID of the service
  DEBUG_BLE_PRINT("Service " + String(service.uuid()));
  // loop the characteristics of the service and explore each
  for (int i = 0; i < service.characteristicCount(); i++) {
    BLECharacteristic characteristic = service.characteristic(i);

    exploreCharacteristic(characteristic);
  }
}

void exploreCharacteristic(BLECharacteristic characteristic) {
  // print the UUID and properties of the characteristic
  Serial.print("\tCharacteristic ");
  Serial.print(characteristic.uuid());
  Serial.print(", properties 0x");
  Serial.print(characteristic.properties(), HEX);

  // check if the characteristic is readable
  if (characteristic.canRead()) {
    // read the characteristic value
    characteristic.read();

    if (characteristic.valueLength() > 0) {
      // print out the value of the characteristic
      Serial.print(", value 0x");
      printData(characteristic.value(), characteristic.valueLength());
    }
  }
  Serial.println();

  // loop the descriptors of the characteristic and explore each
  for (int i = 0; i < characteristic.descriptorCount(); i++) {
    BLEDescriptor descriptor = characteristic.descriptor(i);

    exploreDescriptor(descriptor);
  }
}

void exploreDescriptor(BLEDescriptor descriptor) {
  // print the UUID of the descriptor
  Serial.print("\t\tDescriptor ");
  Serial.print(descriptor.uuid());

  // read the descriptor value
  descriptor.read();

  // print out the value of the descriptor
  Serial.print(", value 0x");
  printData(descriptor.value(), descriptor.valueLength());

  Serial.println();
}

void printData(const unsigned char data[], int length) {
  for (int i = 0; i < length; i++) {
    unsigned char b = data[i];

    if (b < 16) {
      Serial.print("0");
    }

    Serial.print(b, HEX);
  }
}
};
