#pragma once
#include <ArduinoBLE.h>
#include <unordered_map>
#include "issBLE.hpp"

class issBloodPressureMeter : public issModule {
    enum State {
      UNAVAILABLE,
      FOUND,
      CONNECTED,
      SUBSCRIBED,
      UPDATED
    };
    
    issBLE *pBLE;
    BLEDevice mBP;
    State mState = UNAVAILABLE;

    #define MAX_DATA_SIZE 40
    byte mRawData[MAX_DATA_SIZE];

    uint8_t mSystolic;
    uint8_t mDiastolic;
    uint8_t mPulse;

  public:
    issBloodPressureMeter(issBLE *_pBLE) {
      pBLE = _pBLE;
    }

    void reset() {
      if (mState >= FOUND)
        if (mBP.connected()) 
          mBP.disconnect();
      mState = UNAVAILABLE;
    }

    bool loop() {
      bool result = false;
      static BLECharacteristic dataCharacteristic;
      do {
            if (mState == UNAVAILABLE) {
              if (pBLE->findDevice("50:8c:b1:6b:1b:26") == false) break; 
              else mState = FOUND;
            }

            if (mState == FOUND) {
                mBP = pBLE->getCurrentDevice();
                if (!mBP.connect()) {
                    mState = UNAVAILABLE;
                    break;
                }
                else mState = CONNECTED; 
            }

            if (mState == CONNECTED) {
                    if (!mBP.discoverAttributes()) {
                      DEBUG_SPO2_PRINT("Attributes discovery failed");
                      mBP.disconnect();
                      mState = UNAVAILABLE;
                      break;
                    }
                    DEBUG_BP_PRINT("Attributes discovered");
                    delay(1000);
                    // search for service
                    for (int i = 0; i < mBP.serviceCount(); i++) {
                    BLEService service = mBP.service(i);
                    //Serial.println("Explore service " + String(i));
                    //exploreService(service);
                    if (String(service.uuid())  == String("ffe0")) {
                        delay(1000);
                       // search for characteristic
                       for (int i = 0; i < service.characteristicCount(); i++) {                       
                        BLECharacteristic characteristic = service.characteristic(i);
                        //exploreCharacteristic(characteristic);
                        if (String(characteristic.uuid()) == String("ffe1")) {
                          characteristic.subscribe();
                          characteristic.writeValue((byte)0x65);
                              long starttime = millis();
                              while (millis() - starttime < 40000) {
                                if (characteristic.valueUpdated() != 0) {
                                  characteristic.readValue(mRawData, 40);
                                  printRawData();
                                  if (mRawData[0] == 0x67) {
                                    parseData(mRawData);
                                    //Serial.println("GOT DATA!");
                                    //672F3130392F3036362F303634000000000

                                    characteristic.unsubscribe();
                                    mBP.disconnect();
                                    mState = FOUND;
                                    produce();
                                    DEBUG_BP_PRINT(String((int)getSystolic()) + " " + String((int)getDiastolic()) + " " + String((int)getPulse()));                
                                    return true;
                                  }
                                }
                              }
                        }
                      }
                      mState = FOUND;
                      mBP.disconnect();
                      return false;
                    }
                  }    
                  break;   
            }     
            }while (false);

     return result;
    }

    void parseData(byte* data) {
      uint8_t sh = data[2] - 0x30;
      uint8_t sm = data[3] - 0x30;
      uint8_t sl = data[4] - 0x30;

      mSystolic = sh * 100 + sm * 10 + sl;

      uint8_t dh = data[6] - 0x30;
      uint8_t dm = data[7] - 0x30;
      uint8_t dl = data[8] - 0x30;

      mDiastolic = dh * 100 + dm * 10 + dl;

      uint8_t ph = data[10] - 0x30;
      uint8_t pm = data[11] - 0x30;
      uint8_t pl = data[12] - 0x30;

      mPulse = ph * 100 + pm * 10 + pl;
    }
    
    uint8_t getSystolic() {
      return mSystolic;
    }

    uint8_t getDiastolic() {
      return mDiastolic;
    }

    uint8_t getPulse() {
      return mPulse;
    }

    void printRawData() {
      // mRawData: header of 10 bytes, then payload

      // for SPO2 payload[0] = bits0-3 signal strength
      // for SPO2 payload[1] 
      // for SPO2 payload[2]: bit4==1 no finger, bit6 is pulserate msbit
      // for SPO2 payload[3]: bits 0-6 are pulse rate lsbyte
      // for SPO2 payload[4]: bits 0-6 are spo2 (%)
      for (int i = 0; i < 22; i++) {
        Serial.print(mRawData[i], HEX);
      }
      Serial.println();
    }


    String consumeToJSONstring() {
    //"BLOODPRESSURE" : [SISTOLIC, DIASTOLIC, PULS_BPM]
      if (isAvailable()) {consume(); return ",\"BLOODPRESSURE\":[" + 
        String(getSystolic()) + "," + 
        String(getDiastolic()) + "," + 
        String(getPulse()) + "]";
      }
      else return String(""); 
  }
};
