#pragma once
#include <ArduinoBLE.h>
#include <unordered_map>
#include "issBLE.hpp"

class issPulseOximeter : public issModule {
    enum State {
      UNAVAILABLE,
      FOUND,
      CONNECTED,
      SUBSCRIBED,
      UPDATED
    };
    
    issBLE *pBLE;
    BLEDevice mSPO2;
    State mState = UNAVAILABLE;
    byte mRawData[40];

  public:
    issPulseOximeter(issBLE *_pBLE) {
      pBLE = _pBLE;
    }

    void reset() {
      if (mState >= FOUND)
        if (mSPO2.connected()) 
          mSPO2.disconnect();
      mState = UNAVAILABLE;
    }

    bool loop() {
      bool result = false;
      static BLECharacteristic spo2_dataCharacteristic;
      do {
            if (mState == UNAVAILABLE) {
              if (pBLE->findDevice("00:a0:50:04:1a:19") == false) break; 
              else mState = FOUND;
            }

            if (mState == FOUND) {
                mSPO2 = pBLE->getCurrentDevice();
                if (!mSPO2.connect()) {
                    mState = UNAVAILABLE;
                    break;
                }
                else mState = CONNECTED; 
            }

            if (mState == CONNECTED) {
                    if (!mSPO2.discoverAttributes()) {
                    DEBUG_SPO2_PRINT("Attributes discovery failed");
                    mSPO2.disconnect();
                    mState = UNAVAILABLE;
                    break;
                  }
                    DEBUG_SPO2_PRINT("Attributes discovered");
            
                  BLEService spo2_dataService = mSPO2.service("49535343-FE7D-4AE5-8FA9-9FAFD205E455");
                  if (!spo2_dataService) {
                      DEBUG_SPO2_PRINT("Data service not found");
                    break;
                  }
                  DEBUG_SPO2_PRINT("Data service found");
            
                  spo2_dataCharacteristic = spo2_dataService.characteristic("49535343-1e4d-4bd9-ba61-23c647249616");
                  if (!spo2_dataCharacteristic) {
                    DEBUG_SPO2_PRINT("Data characteristic not found");
                    break;
                  }
                  DEBUG_SPO2_PRINT("Data characteristic found");
            
                  if (!spo2_dataCharacteristic.subscribe()) {
                    DEBUG_SPO2_PRINT("Could not subscribe to data characteristic");
                    break;
                  }
                  DEBUG_SPO2_PRINT("Subscribed to data characteristic!");
                  mState = SUBSCRIBED;
            }

            if (mState == SUBSCRIBED) {  
              if (!mSPO2.connected()) {mState = UNAVAILABLE; break;}       
              DEBUG_SPO2_PRINT("valueSubscribed in");
              long starttime = millis();
              while (millis() - starttime < 1000) {
                if (spo2_dataCharacteristic.valueUpdated() != 0) {mState = UPDATED; break;}
              }
              if (mState != UPDATED) {mState = FOUND;}
              DEBUG_SPO2_PRINT("valueSubscribed out");
            }           

            if (mState == UPDATED) {
              long starttime = millis();
              while (millis() - starttime < 1000) {
                spo2_dataCharacteristic.readValue(mRawData, 20);
                if (((getSaturation() > 40) && (getSaturation() <= 100)) && 
                   (getPulse() < 100) && (getPulse() > 40)) 
                      {produce(); mState = UPDATED;break;}
                else {
                    spo2_dataCharacteristic.unsubscribe();
                    mSPO2.disconnect();
                    mState = FOUND;
                   }
                //printRawData();                
              }

              if (mState == UPDATED) {
                DEBUG_SPO2_PRINT(String((int)getSaturation()) + " " + String((int)getPulse()));                
                mState = SUBSCRIBED;
                result = true;
                break;
              }
              else {
                mSPO2.disconnect();
                break;
              }
            }
      }while (false);

     return result;
    }
    
    uint8_t getPulse() {
        uint8_t pulse_spo2 = 0;
        uint8_t pulse_low = mRawData[13];
        pulse_low &= 0b01111111;

        uint8_t pulse_high = mRawData[12];
        pulse_high &= 0b01000000;

        if (pulse_high == 0)
          pulse_spo2 = pulse_low;

        if (pulse_high == 0b01000000)
          pulse_spo2 = pulse_low + 0b10000000;

       return pulse_spo2;
    }

    uint8_t getSaturation() {
        uint8_t spo2 = mRawData[14];
        spo2 &= 0b01111111;
        return spo2;
    }

    void printRawData() {
      // mRawData: header of 10 bytes, then payload

      // for SPO2 payload[0] = bits0-3 signal strength
      // for SPO2 payload[1] 
      // for SPO2 payload[2]: bit4==1 no finger, bit6 is pulserate msbit
      // for SPO2 payload[3]: bits 0-6 are pulse rate lsbyte
      // for SPO2 payload[4]: bits 0-6 are spo2 (%)
      for (int i = 0; i < 20; i++) {
        Serial.print(mRawData[i], HEX);
      }
      Serial.println();
    }

    //"SPO2" : [PULS_BPM, SATURATIE_IN_PROCENTE]
    String consumeToJSONstring() {
      if (isAvailable()) {consume(); return (",\"SPO2\":[" + String(getPulse()) + "," + String(getSaturation())) + "]";}
      else return String(""); 
    }
};
