/*
 *  Arduino library used to handle a WiFi client used for ISS WellBeing system
 *  Version _1.0
 *  
 *  @author Costin Vasile (costin.vasile1003@upb.ro)
 */
#pragma once
#include <SPI.h>
#include <WiFiNINA.h>

class issWebClient{
 private:
  char* mSSID;
  char* mPASS;
  char* mServer;
  
  WiFiClient mClient;

 public:
  issWebClient(char* server, char* ssid, char* pass){
    mServer = server;
    mSSID = ssid;
    mPASS = pass;
  }

  void init(){
    uint8_t tries = 0;
    int status = WL_IDLE_STATUS;
    while(status != WL_CONNECTED){
      if(tries > 5){
        Serial.print("[WARNING] Unable to connect to SSID: ");
        Serial.print(mSSID);
        Serial.println(" | WiFi functionality will not be available!");
        return;
      }
      Serial.print("[INFO] Attempting to connect to SSID: ");
      Serial.println(mSSID);
      status = WiFi.begin(mSSID, mPASS);
      tries++;
      delay(5000*tries);
    }
    Serial.println("[INFO] Connected to WiFi!");
//    printWifiStatus();
  }
  
  bool do_POST(const char* body, uint8_t body_size, char* path, char* api_key, uint8_t port){
    init();
    if(WiFi.status() != WL_CONNECTED){
      Serial.println("[WARNING] WiFi not connected. Unable to send data to cloud");
      WiFi.end();
      return false;
    }
    Serial.print("[INFO] Connecting to server... ");
    if(!mClient.connect(mServer, port)){
      Serial.println("\n[WARNING] Connection failed! Data not sent");
      WiFi.end();
      return false;
    }
    Serial.println ("connection done");

    //Serial.println();
    mClient.println("POST " + String(path) + " HTTP/1.0"); //Serial.println("POST " + String(path) + " HTTP/1.0");
    mClient.println("x-api-key: " + String(api_key)); //Serial.println("x-api-key: " + String(api_key));
    mClient.println("Content-Type: application/json"); //Serial.println("Content-Type: application/json");
    mClient.println("Content-Length: " + String(body_size)); //Serial.println("Content-Length: " + String(body_size));
    //mClient.println("Host: www.example.com"); //Serial.println("Host: www.example.com");
    mClient.println(); //Serial.println();
    mClient.println(String(body)); //Serial.println(String(body));
    mClient.println(); //Serial.println();
    //Serial.println();

    delay(2000);
    char* answer = (char*) malloc(13*sizeof(char));
    answer[0] = 0;
    for(int i = 0; i < 12; i++){
      if(!mClient.available()){
        Serial.println("[WARNING] No answer from server. Data may not be sent!");
        WiFi.end();
        return false;
      }
      answer[i] = mClient.read();
    }

    answer[12] = 0;
    if(0 != strcmp(answer, "HTTP/1.1 200")){
      Serial.println("[WARNING] The server answered, but with no succes response. Data may not be sent!");
      Serial.println(answer);
      WiFi.end();
      return false;
    } 
    else Serial.println("[INFO] Data sent ok to cloud");
    WiFi.end();
    return true;
  }

  void printWifiStatus() {
    Serial.print("[INFO] SSID: ");
    Serial.println(WiFi.SSID());
    
    IPAddress ip = WiFi.localIP();
    Serial.print("[INFO] IP Address: ");
    Serial.println(ip);
    
    long rssi = WiFi.RSSI();
    Serial.print("[INFO] Signal strength (RSSI):");
    Serial.print(rssi);
    Serial.println(" dBm");
  }
};
