#pragma once
/**
 * Class implementing battery status for main board
 * Assumes direct correlation between SOC and open-voltage 
 * 
 * @author Calin Bira (calin.bira@upb.ro)
 */
class issBattery {
  // 0     1         20
  // 100%, 95%, ..., 0%
  // Percent = 5*(20 - i);
  #define SOC_VALUES 21
  static const int BatteryVoltage_vs_SOC[SOC_VALUES];
  
  public:

  float getVoltage_zmV() {
    int sensorValue = analogRead(ADC_BATTERY);
    // 1M2 resistor to ground, 330k resistor to Vbat
    return sensorValue * (420.0 / 1023.0);
  }

  int getPercentFromIndex(int i) {
      return 100*((SOC_VALUES-1)-i) / (SOC_VALUES-1);
  }

  int getBatteryPercent() {
    
    int zmV = getVoltage_zmV();
    //Serial.print(zmV);

    int Percent = 100;
    int i = 0;
    for (; i < SOC_VALUES; i++)
      if (zmV < BatteryVoltage_vs_SOC[i]) 
          Percent = getPercentFromIndex(i);
    
    return Percent;
  }
};

  /**
   * Table with voltage vs battery SOC (index is 100%, 95% ...) and value is in tens of millivolts (420 == 4200mV)
   */
  const int issBattery::BatteryVoltage_vs_SOC[SOC_VALUES] = {
    420, 415, 411, 408, 402, 398, 395, 391, 387, 385,
    384, 382, 380, 379, 377, 375, 373, 371, 369, 361, 
    327 
  };
