#pragma once

class issLED {

  protected:
    int mPinled;
    int mDelayOn_ms;
    int mDelayOff_ms;

  public:
  issLED(int led, int delayOn_ms, int delayOff_ms) {
    mDelayOn_ms = delayOn_ms;
    mDelayOff_ms = delayOff_ms;
    mPinled = led;
    pinMode(mPinled, OUTPUT);
  }

  virtual ~issLED() {}

  virtual void blink(int msOn = -1, int msOff = -1, bool b = false) {        
      if (msOn != -1) mDelayOn_ms = msOn;
      if (msOff != -1) mDelayOff_ms = msOff;

      do {
        digitalWrite(mPinled, LED_CRT_ON);
        delay(mDelayOn_ms);
        digitalWrite(mPinled, LED_CRT_OFF);
        delay(mDelayOff_ms);
      } while (b == true);
  }

  virtual void switchOff() {digitalWrite(mPinled, LED_CRT_OFF);}
  virtual void switchOn() {digitalWrite(mPinled, LED_CRT_ON);}
};


class issSmartLED : public issLED {

  enum SmartLedState {
    LED_IDLE = 0,
    LED_ON = 1,
    LED_OFF = 2,
    LED_BLINK_STOPPED = 3
  };

  volatile SmartLedState mState;
  volatile unsigned long mlastActionTimestamp;
  
  public:
    issSmartLED(int led, int delayOn_ms, int delayOff_ms) : issLED(led, delayOn_ms, delayOff_ms) {
      mState = LED_IDLE;
    }

  virtual void switchOn() override {
      mState = LED_BLINK_STOPPED;
      issLED::switchOn();
  }

  virtual void switchOff() override {
      mState = LED_BLINK_STOPPED;
      issLED::switchOff();
  }

  virtual void blink(int msOn = -1, int msOff = -1, bool restart = false) override {
      if (msOn != -1) mDelayOn_ms = msOn;
      if (msOff != -1) mDelayOff_ms = msOff;
      
      if ((restart == true) && (mState == LED_BLINK_STOPPED)) mState = LED_IDLE;
      
      switch (mState) {
        case LED_BLINK_STOPPED: break;
        case LED_IDLE: {
            //Serial.println(String(mlastActionTimestamp) + " " + String(mPinled) + " " + String(mState) + " " + String(mDelayOn_ms));
            issLED::switchOn();
            mState = LED_ON;
            mlastActionTimestamp = millis();
            break;
        }
        
        case LED_ON: {
            if (millis()-mlastActionTimestamp > (unsigned)mDelayOn_ms) {
              //Serial.println(String(mlastActionTimestamp) + " " + String(mPinled) + " " + String(mState)+ " " + String(mDelayOn_ms));
              issLED::switchOff();
              mState = LED_OFF;
              mlastActionTimestamp = millis();
            }
            break;
        }

        case LED_OFF: {
            if (millis()-mlastActionTimestamp > (unsigned)mDelayOff_ms) {
              //Serial.println(String(mlastActionTimestamp) + " " + String(mPinled) + " " + String(mState)+ " " + String(mDelayOn_ms));
              if (restart == true)
                mState = LED_IDLE;
              else
                mState = LED_BLINK_STOPPED;
            }
            break;
        }
    }
  }
  
};
