#pragma once

class issTimerTasks {
  int TimerTasksReloadValues[MAX_TIMERS];
  int TimerTasksCurrentValues[MAX_TIMERS];
  int TimerTasksRunning;
  int TimerTasksOverflow;
  long StartMiliseconds;
  
  static issTimerTasks* pInstance;
  static int TimerIndex;
  
  issTimerTasks() { 
    TimerIndex = 0;
    TimerTasksRunning = 0; 
    StartMiliseconds = millis();
    for (int i = 0; i < MAX_TIMERS; i++) {
      TimerTasksReloadValues[i] = 0;
      TimerTasksCurrentValues[i] = 0;
    }
  }
  
  public:
    static int getNextTimer() {
      return TimerIndex++;
    }
    
    static issTimerTasks* getInstance() {
      if (pInstance == NULL) pInstance = new issTimerTasks();      
      return pInstance;
    }
    
    void setTimerTask(int id, int millisecs) {TimerTasksReloadValues[id] = millisecs;}
    void restartTimerTask(int id) {TimerTasksCurrentValues[id] = TimerTasksReloadValues[id]; startTimerTask(id);}
    void startTimerTask(int id) {TimerTasksRunning |= (1 << id);}
    void stopTimerTask(int id) {TimerTasksRunning &= ~(1 << id);}
    bool isOverflown(int id) { return TimerTasksOverflow & (1 << id); }
    bool clearOverflow(int id) { TimerTasksOverflow &= ~(1 << id);}
    bool isRunning(int id) {return TimerTasksRunning & (1 << id);}
    void printAll() {
      Serial.println();
      Serial.println("<SYS>Timer_number  Running Overflow CurrentValue ReloadValue"); 
      for (int i = 0; i < MAX_TIMERS; i++) {
        Serial.println("<SYS>Timer " + String(i) + ":\t" + (isRunning(i)?"true":"false") + "\t" + 
          (isOverflown(i)?"true":"false") + "\t" + 
           String(TimerTasksCurrentValues[i]) + "\t" + String(TimerTasksReloadValues[i]));
      }
    }
    
    void loop() {
      if (millis() - StartMiliseconds < 1) return;
      StartMiliseconds = millis();
      if (TimerTasksRunning) {
        for (int i = 0; i < MAX_TIMERS; i++)
          if (isRunning(i))
            if (--TimerTasksCurrentValues[i] == 0) {
              TimerTasksOverflow |= (1 << i);    
              TimerTasksCurrentValues[i] = TimerTasksReloadValues[i];
            }
      }
    }  
};

issTimerTasks* issTimerTasks::pInstance = NULL;
int issTimerTasks::TimerIndex = 0;

class issTimerTask {
  int mIndex;
  public:
    issTimerTask(int ms) {
      mIndex = issTimerTasks::getInstance()->getNextTimer();
      setTimerTask(ms);
    }
    bool isRunning()          { return issTimerTasks::getInstance()->isRunning(mIndex);}
    void setTimerTask(int ms) { return issTimerTasks::getInstance()->setTimerTask(mIndex, ms);}
    void startTimerTask()     { return issTimerTasks::getInstance()->startTimerTask(mIndex);}
    void restartTimerTask()   { return issTimerTasks::getInstance()->restartTimerTask(mIndex);}
    void stopTimerTask()      { return issTimerTasks::getInstance()->stopTimerTask(mIndex);}
    bool isOverflown()        { return issTimerTasks::getInstance()->isOverflown(mIndex);}
    bool clearOverflow()      { return issTimerTasks::getInstance()->clearOverflow(mIndex);}    
    int getIndex() {return mIndex;}
    
    static void swait(issTimerTask* tt) {
      tt->restartTimerTask();
      while (!tt->isOverflown()) 
        issTimerTasks::getInstance()->loop(); // wait for ovf
      tt->clearOverflow();
      tt->stopTimerTask();
    }
    
    void mwait() {
      restartTimerTask();
      while (!isOverflown()) 
        issTimerTasks::getInstance()->loop(); // wait for ovf
      clearOverflow();
      stopTimerTask();
    }
};
