#pragma once
#include "Adafruit_BLEBattery.h"

class issWxBattery {
  #define SOC_STEPS 12
  uint16_t SOC[SOC_STEPS]= {0,   500,    1000,   2000,   3000,   4000,   5000,   6000,   7000,   8000,   9000,  10000};
  uint16_t  VSOC[SOC_STEPS] = {3000, 3450, 3680, 3740, 3770, 3790, 3820, 3870, 3920, 3980, 4060, 4200};

  uint32_t mLastMilliseconds;
  float mLastSOC;  
  float mSOC;
  
  uint16_t mMillivolts;
  int16_t mEnduranceMinutes;
  
  Adafruit_BLEBattery *pblebatt = NULL;
  public:
  issWxBattery() {}

  void init(Adafruit_BLE* pble) {
    if (pble!=NULL) {
      pblebatt = new Adafruit_BLEBattery(*pble);
      pblebatt->begin(true);
    }

    for (uint16_t voltage = 4200; voltage >= 3000; voltage = voltage - 10) {
      SERIAL_PRINT(voltage);
      SERIAL_PRINT("mV => ");
      SERIAL_PRINTLN(getPercentFromVoltage(voltage));
    }

    mLastMilliseconds = millis();
    mSOC = getSOC();
    mLastSOC = mSOC;
    mEnduranceMinutes = 0;
  }

  uint16_t getMillivolts() {
    return mMillivolts;
  }
  
  float getSOC() {
    // Voltage divider resistor values.
    const float R1 = 10000;
    const float R2 = 10000;
    // Voltage divider formula.
    const float Divider = (R2 / (R1 + R2));  
    
    static byte bPin = A11;
    static byte battEn = 4;
    uint8_t BatteryPercent;
  
    digitalWrite(battEn, HIGH);
    delay(50);

    float voltage = analogRead(bPin);
    
    voltage = (voltage / 1024) * 3.35;
    voltage = voltage / Divider;
    mMillivolts = voltage * 1000;
    
    delay(50);
    digitalWrite(battEn, LOW);

    SERIAL_PRINT("Battery Voltage: ");
    SERIAL_PRINT(mMillivolts);
    SERIAL_PRINTLN("mV");
    
    return getPercentFromVoltage(mMillivolts);
  }

  float getPercentFromVoltage(uint16_t millivolts) {
    float BatteryPercent;
    if (millivolts <= VSOC[0]) BatteryPercent = 0;
    else if (millivolts >= VSOC[SOC_STEPS-1]) BatteryPercent = 100;
    else {
      uint8_t i = 1;
      for (; i < SOC_STEPS; i++)
        if (millivolts < VSOC[i]) break;
      
      //map(value, fromLow, fromHigh, toLow, toHigh)
      BatteryPercent = map(millivolts, VSOC[i-1], VSOC[i], SOC[i-1], SOC[i]) / 100.0;
    }
    return BatteryPercent;      
  }

  void loop() {
    float retval = 0;    
    mSOC = getSOC();
    //if (pblebatt != NULL) pblebatt->update(mSOC);
    if (pblebatt != NULL) pblebatt->update(99);
    uint32_t currentMilliseconds = millis();
            
    if (abs(mLastSOC - mSOC) >= 3) {
      if (mSOC > mLastSOC) // charging
        mEnduranceMinutes = (100 - mSOC) * (currentMilliseconds - mLastMilliseconds) / (mLastSOC - mSOC) / 60000.;
      else // discharging
        mEnduranceMinutes = (mSOC * (currentMilliseconds - mLastMilliseconds) / (mLastSOC - mSOC)) / 60000.;

      mLastMilliseconds = currentMilliseconds;
      mLastSOC = mSOC;
    }
  }
  
  int16_t getEnduranceMinutes() {
    return mEnduranceMinutes;
  }
};
