#pragma once

#include <Arduino.h>
#include <SPI.h>
#include "Adafruit_BLE.h"
#include "Adafruit_BluefruitLE_SPI.h"
#include "Adafruit_BluefruitLE_UART.h"
#include "iss_wx_ble_config.h"

#if SOFTWARE_SERIAL_AVAILABLE
  #include <SoftwareSerial.h>
#endif

class issWxBLE {
  /*********************************************************************
 MIT license, check LICENSE for more information
 All text above, and the splash screen below must be included in
 any redistribution
*********************************************************************/

/*=========================================================================
    APPLICATION SETTINGS

    FACTORYRESET_ENABLE       Perform a factory reset when running this sketch
   
                              Enabling this will put your Bluefruit LE module
                              in a 'known good' state and clear any config
                              data set in previous sketches or projects, so
                              running this at least once is a good idea.
   
                              When deploying your project, however, you will
                              want to disable factory reset by setting this
                              value to 0.  If you are making changes to your
                              Bluefruit LE device via AT commands, and those
                              changes aren't persisting across resets, this
                              is the reason why.  Factory reset will erase
                              the non-volatile memory where config data is
                              stored, setting it back to factory default
                              values.
       
                              Some sketches that require you to bond to a
                              central device (HID mouse, keyboard, etc.)
                              won't work at all with this feature enabled
                              since the factory reset will clear all of the
                              bonding data stored on the chip, meaning the
                              central device won't be able to reconnect.
    MINIMUM_FIRMWARE_VERSION  Minimum firmware version to have some new features
    MODE_LED_BEHAVIOUR        LED activity, valid options are
                              "DISABLE" or "MODE" or "BLEUART" or
                              "HWUART"  or "SPI"  or "MANUAL"
    -----------------------------------------------------------------------*/
    #define FACTORYRESET_ENABLE         1
    #define MINIMUM_FIRMWARE_VERSION    "0.6.6"
    #define MODE_LED_BEHAVIOUR          "MODE"
/*=========================================================================*/

Adafruit_BluefruitLE_SPI* pble;

public:
  Adafruit_BLE* getBLEhandle() {
    return pble;
  }
  
  issWxBLE() {
    // Create the bluefruit object, either software serial...uncomment these lines
    /*
    SoftwareSerial bluefruitSS = SoftwareSerial(BLUEFRUIT_SWUART_TXD_PIN, BLUEFRUIT_SWUART_RXD_PIN);
    
    Adafruit_BluefruitLE_UART ble(bluefruitSS, BLUEFRUIT_UART_MODE_PIN,
                          BLUEFRUIT_UART_CTS_PIN, BLUEFRUIT_UART_RTS_PIN);
    */
  
    /* ...or hardware serial, which does not need the RTS/CTS pins. Uncomment this line */
    // Adafruit_BluefruitLE_UART ble(Serial1, BLUEFRUIT_UART_MODE_PIN);
    
    /* ...hardware SPI, using SCK/MOSI/MISO hardware SPI pins and then user selected CS/IRQ/RST */
    pble = new Adafruit_BluefruitLE_SPI(BLUEFRUIT_SPI_CS, BLUEFRUIT_SPI_IRQ, BLUEFRUIT_SPI_RST);
    
    /* ...software SPI, using SCK/MOSI/MISO user-defined SPI pins and then user selected CS/IRQ/RST */
    //Adafruit_BluefruitLE_SPI ble(BLUEFRUIT_SPI_SCK, BLUEFRUIT_SPI_MISO,
    //                             BLUEFRUIT_SPI_MOSI, BLUEFRUIT_SPI_CS,
    //                             BLUEFRUIT_SPI_IRQ, BLUEFRUIT_SPI_RST);
  }
  
  ~issWxBLE() {
    delete(pble);
  }
  
  /**************************************************************************/
  /*!
      @brief  Sets up the HW an the BLE module (this function is called
              automatically on startup)
  */
  /**************************************************************************/

  void init(void)
  {
    if (pble == NULL) return;
    
    //while (!Serial);  // required for Flora & Micro
    delay(500);
  
    Serial.begin(115200);
    SERIAL_PRINTLN(F("iss BLE Command Mode "));
    SERIAL_PRINTLN(F("---------------------------------------"));
  
    /* Initialise the module */
    SERIAL_PRINT(F("Initialising the Bluefruit LE module: "));
  
    if ( !pble->begin(VERBOSE_MODE) )
    {
      error(F("Couldn't find Bluefruit, make sure it's in CoMmanD mode & check wiring?"));
    }
    SERIAL_PRINTLN( F("OK!") );
  
    if ( FACTORYRESET_ENABLE )
    {
      /* Perform a factory reset to make sure everything is in a known state */
      SERIAL_PRINTLN(F("Performing a factory reset: "));
      if ( ! pble->factoryReset() ){
        error(F("Couldn't factory reset"));
      }
    }
  
    /* Disable command echo from Bluefruit */
    pble->echo(false);
  
    SERIAL_PRINTLN("Requesting Bluefruit info:");
    /* Print Bluefruit information */
    pble->info();
  
    SERIAL_PRINTLN(F("Please use Adafruit Bluefruit LE app to connect in UART mode"));
    SERIAL_PRINTLN(F("Then Enter characters to send to Bluefruit"));
    SERIAL_PRINTLN();
  
    pble->verbose(false);  // debug info is a little annoying after this point!
  
    /* Wait for connection */
    /*
    while (! pble->isConnected()) {
        delay(500);
    }
  
    // LED Activity command is only supported from 0.6.6
    if ( pble->isVersionAtLeast(MINIMUM_FIRMWARE_VERSION) )
    {
      // Change Mode LED Activity
      Serial.println(F("******************************"));
      Serial.println(F("Change LED activity to " MODE_LED_BEHAVIOUR));
      pble->sendCommandCheckOK("AT+HWModeLED=" MODE_LED_BEHAVIOUR);
      Serial.println(F("******************************"));
    }
    */
  }

/**************************************************************************/
/*!
    @brief  Constantly poll for new command or response data
*/
/**************************************************************************/

private:
  // A small helper
  void error(const __FlashStringHelper*err) {
    SERIAL_PRINTLN(err);
    while (1);
  }
  
  void loopUARTBLE(void)
  {
    // Check for user input
    char inputs[BUFSIZE+1];
  
    if ( getUserInput(inputs, BUFSIZE) )
    {
      // Send characters to Bluefruit
      SERIAL_PRINT("[Send] ");
      SERIAL_PRINTLN(inputs);
  
      pble->print("AT+BLEUARTTX=");
      pble->println(inputs);
  
      // check response stastus
      if (! pble->waitForOK() ) {
        SERIAL_PRINTLN(F("Failed to send?"));
      }
    }
  
    // Check for incoming characters from Bluefruit
    pble->println("AT+BLEUARTRX");
    pble->readline();
    if (strcmp(pble->buffer, "OK") == 0) {
      // no data
      return;
    }
    // Some data was found, its in the buffer
    SERIAL_PRINT(F("[Recv] ")); 
    SERIAL_PRINTLN(pble->buffer);
    pble->waitForOK();
  }
  
  /**************************************************************************/
  /*!
      @brief  Checks for user input (via the Serial Monitor)
  */
  /**************************************************************************/
  bool getUserInput(char buffer[], uint8_t maxSize)
  {
    // timeout in 100 milliseconds
    TimeoutTimer timeout(100);
  
    memset(buffer, 0, maxSize);
    while( (!Serial.available()) && !timeout.expired() ) { delay(1); }
  
    if ( timeout.expired() ) return false;
  
    delay(2);
    uint8_t count=0;
    do
    {
      count += Serial.readBytes(buffer+count, maxSize);
      delay(2);
    } while( (count < maxSize) && (Serial.available()) );
  
    return true;
  }
};
