/**
 * ISS Copyright 2019-2020
 * @author Calin Bira (calin.bira@spacescience.ro / calin.bira@upb.ro)
 */
#include <MySignals_BLE.h>

class issBLE {
  bool mInitialized = false;
  issDisplay *pDisplay = NULL;

  #define MAX_BLE_DEVICES 10
  char* mBleMACs[MAX_BLE_DEVICES];
  bool mBleAvailable[MAX_BLE_DEVICES];
  uint8_t mNumBleMACs = 0;
  
  public:
    bool isInitialized() {
      return mInitialized;
    }
    
    void deinit() {
      mInitialized = false;
      bitClear(MySignals.expanderState, EXP_BLE_POWER);
      MySignals.expanderWrite(MySignals.expanderState);
    }

    void reset() {
      MySignals_BLE.hardwareReset();
      MySignals_BLE.initialize_BLE_values();
      
      delay(100);
    }
    
    void init(issDisplay *_pDisplay) {
      if (mInitialized) return;
      issDisplay *pDisplay = _pDisplay;
      
      MySignals.enableSensorUART(BLE);
    
      //Enable BLE module power -> bit6: 1
      bitSet(MySignals.expanderState, EXP_BLE_POWER);
      MySignals.expanderWrite(MySignals.expanderState);
    
      //Enable BLE UART flow control -> bit5: 0
      bitClear(MySignals.expanderState, EXP_BLE_FLOW_CONTROL);
      MySignals.expanderWrite(MySignals.expanderState);    
    
      MySignals_BLE.hardwareReset();    
      MySignals_BLE.initialize_BLE_values();

      do{
          if (MySignals_BLE.initModule() != 1) {
            #ifdef BLE_APP_DEBUG_MESSAGES
              if (pDisplay != NULL) 
                pDisplay->drawString("BLE init1 failed");
            #endif
            break;
          }

          if (MySignals_BLE.sayHello() != 1) {
            #ifdef BLE_APP_DEBUG_MESSAGES
              if (pDisplay != NULL) pDisplay->drawString("BLE init2 failed");
            #endif
            break;
          }

          #ifdef BLE_APP_DEBUG_MESSAGES
            if (pDisplay != NULL)
              pDisplay->drawString("BLE init ok");
          #endif  
          mInitialized = true;
      } while (0);
};
  
    bool pushMAC(char *mac) {
      for (uint8_t i = 0; i < mNumBleMACs; i++)
        if (mBleMACs[i] == mac) return true; // compare pointers!
        
      if (mNumBleMACs < MAX_BLE_DEVICES) {
        mBleMACs[mNumBleMACs] = mac;
        mBleAvailable[mNumBleMACs] = false;
        mNumBleMACs++;
        return true;
      }    
      else return false;
    }

    bool hasfoundMAC(char* mac) {
      for (uint8_t i = 0; i < mNumBleMACs; i++)
        if (mBleMACs[i] == mac) return mBleAvailable[i]; // compare pointers!
      return false;
    }

    bool enumerate() {
      bool ret = false;
      for (uint8_t i = 0; i < mNumBleMACs; i++) {
        mBleAvailable[i] = MySignals_BLE.scanDevice(mBleMACs[i], 1000, TX_POWER_MAX);
        ret |= mBleAvailable[i]; 
      }
      return ret;
    }
};
