/**
 * ISS Copyright 2019-2020
 * @author Calin Bira (calin.bira@spacescience.ro / calin.bira@upb.ro)
 */
 
#pragma once
#include <Adafruit_GFX_AS.h>
#include <Adafruit_ILI9341_AS.h>

class issBloodPressure : issModule {

private:
  char *MAC_BP;
  
  uint8_t available_bp = 0;
  uint8_t connected_bp = 0;
  uint8_t connection_handle_bp = 0;
  
  #define BP_HANDLE 18
  
public:
  struct bloodPressureBLEDataVector
  {
    uint16_t systolic;
    uint16_t diastolic;
    uint16_t pulse;
  };
private:
  bloodPressureBLEDataVector bloodPressureBLEData;
  issBLE *pBLE;
  
public:
  issBloodPressure(char *mac) {
    MAC_BP = mac;
  }
  
  char* getMac() {
    return MAC_BP;
  }
        
  void init(issDisplay *_pDisplay, issBLE *_pBLE) 
  {
      pBLE = _pBLE;
      pDisplay = _pDisplay;

      if (! pBLE->isInitialized())
        pBLE->init(_pDisplay);
      pBLE->pushMAC(MAC_BP);
      
      mInitialized = true;
      
      bloodPressureBLEData.systolic = 0;
      bloodPressureBLEData.diastolic = 0;
      bloodPressureBLEData.pulse = 0;
  }

  void loop()
  {
    #ifndef USE_DUMMY_DATA
      if (pBLE) pBLE->reset();
      available_bp = MySignals_BLE.scanDevice(MAC_BP, 1000, TX_POWER_MAX);
      //if (pDisplay != NULL) pDisplay->drawString("BP available: " + String(available_bp));
    
      if (available_bp == 1)
      {
        if (MySignals_BLE.connectDirect(MAC_BP) == 1)
        {
          connected_bp = 1;
          if (pDisplay != NULL) pDisplay->drawString("Connected " + String(connected_bp));
  
          delay(8000);
          //To subscribe the BP measure write an ASCII letter "e"
          if (MySignals_BLE.attributeWrite(MySignals_BLE.connection_handle,  BP_HANDLE, "e", 1) == 0)
          {  
            if (pDisplay != NULL) pDisplay->drawString("Subscribed");          
            unsigned long previous = millis();
            do
            {  
              if (MySignals_BLE.waitEvent(1000) == BLE_EVENT_ATTCLIENT_ATTRIBUTE_VALUE)
              {
                //Search letter "g" in ASCII
                if (MySignals_BLE.event[9] == 0x67)
                {  
                  if (MySignals_BLE.event[10] == 0x2f)
                  {
                    //Ojo esto esta mal en la guia de comandos
    
                    //Primero da la alta -> sistolica
                    uint8_t sh = MySignals_BLE.event[11] - 48;
                    uint8_t sm = MySignals_BLE.event[12] - 48;
                    uint8_t sl = MySignals_BLE.event[13] - 48;
                    bloodPressureBLEData.systolic = (sh * 100) + (sm * 10) + sl;
    
                    //Primero da la baja -> diastolica
                    uint8_t dh = MySignals_BLE.event[15] - 48;
                    uint8_t dm = MySignals_BLE.event[16] - 48;
                    uint8_t dl = MySignals_BLE.event[17] - 48;
                    bloodPressureBLEData.diastolic = (dh * 100) + (dm * 10) + dl;
    
                    uint8_t ph = MySignals_BLE.event[19] - 48;
                    uint8_t pm = MySignals_BLE.event[20] - 48;
                    uint8_t pl = MySignals_BLE.event[21] - 48;
                    bloodPressureBLEData.pulse = (ph * 100) + (pm * 10) + pl;
                    produce();
                    produce();
    
                    if (pDisplay != NULL) pDisplay->drawString(String(bloodPressureBLEData.systolic) + " " + 
                      String(bloodPressureBLEData.diastolic) + " " + String(bloodPressureBLEData.pulse));
                    connected_bp = 0;  
                  }
                }
              }
            }
            while ((connected_bp == 1) && ((millis() - previous) < 40000));
            MySignals_BLE.attributeWrite(MySignals_BLE.connection_handle,  BP_HANDLE, "i" , 1);
            delay(100);  
    
            MySignals_BLE.disconnect(MySignals_BLE.connection_handle);
            connected_bp = 0;
          }
          else
          {
            if (pDisplay != NULL) pDisplay->drawString("Error subscribing");
          }
    
        }
        else
        {
          connected_bp = 0;
          if (pDisplay != NULL) pDisplay->drawString("Not Connected");
        }
      }
      else if (available_bp == 0)
      {
        //Do nothing
      }
      else
      {
        MySignals_BLE.hardwareReset();
        MySignals_BLE.initialize_BLE_values();
        delay(100);
      }
    #else
      // "BLOODPRESSURE" : [SISTOLIC, DIASTOLIC, PULS_BPM], // SISTOLIC este int16 numar intre 0-999 mm Hg, DIASTOLIC 0-999 mm Hg, PULS_BPM 0-999 bpm
  
      bloodPressureBLEData.systolic = (bloodPressureBLEData.systolic + 1) % 30 + 50;
      bloodPressureBLEData.diastolic = (bloodPressureBLEData.diastolic + 1) % 30 + 90;
      bloodPressureBLEData.pulse = (bloodPressureBLEData.pulse + 1) % 160 + 50;
      produce();
      produce();

    #endif
  }

  String consumeToJSONstring() {
    //"BLOODPRESSURE" : [SISTOLIC, DIASTOLIC, PULS_BPM]
      if (isAvailable()) {consume(); return ",\"BLOODPRESSURE\":[" + 
        String(bloodPressureBLEData.systolic) + "," + 
        String(bloodPressureBLEData.diastolic) + "," + 
        String(bloodPressureBLEData.pulse) + "]";
      }
      else return String(""); 
  }

  String consumeToDisplayedString() {
    if (isAvailable()) {consume(); 
                        return " BLOODPRESSURE " + String(bloodPressureBLEData.systolic) + "," + 
                                                    String(bloodPressureBLEData.diastolic) + "," + 
                                                    String(bloodPressureBLEData.pulse) + getEmptyString();}
    else return " BLOODPRESSURE" + getEmptyString() + getEmptyString();
  }

  
    void show() {
      if (pDisplay != NULL) {
          if (!isAvailable()) {pDisplay->drawString("Bloodpressure: n/a"); return;}

          pDisplay->drawString("Bloodpressure:");
          pDisplay->drawString("Systolic:" + String(bloodPressureBLEData.systolic));
          pDisplay->drawString("Diastolic:" + String(bloodPressureBLEData.diastolic));
          pDisplay->drawString("Pulse:" + String(bloodPressureBLEData.pulse));
        }
    }
};
