/**
 * ISS Copyright 2019-2020
 * @author Calin Bira (calin.bira@spacescience.ro / calin.bira@upb.ro)
 */
 
#pragma once
class issBodyPositionSensor : public issModule {

uint8_t mPosition = 0;

public:
  void init(issDisplay *_pDisplay) 
  {
      MySignals.initBodyPosition();
      pDisplay = _pDisplay;
      mInitialized = true;
      mPosition = 0;
  }

  String positionText[7] = {
    "n/a", //0
    "Prone position ",  // 1
    "Left lateral   ",  // 2
    "Right lateral  ",  // 3
    "Supine position",  // 4
    "Stand or sit   ",  // 5
    "non-defined    "   // 6
  };

  enum BodyPositions {
    NOT_AVAILABLE       = 0,
    PRONE_POS           = 1,
    LEFT_LATERAL_POS    = 2,
    RIGHT_LATERAL_POS   = 3,
    SUPINE_LATERAL_POS  = 4,
    STAND_OR_SIT_POS    = 5,
    NON_DEFINED_POS     = 6
  };

  String getPositionName(uint8_t val) {
    return positionText[val];
  }
  
  void loop()
  {
    #ifndef USE_DUMMY_DATA
    int Position = getBodyPosition(); 
    if ((Position > NOT_AVAILABLE) && (Position < NON_DEFINED_POS)) 
    {
      mPosition = Position;
      produce();
      produce();
    }
    SPI.end();
    #else
      mPosition = (mPosition % 6) + 1;
      produce();
      produce();
    #endif
  };

  void show() {
    if (pDisplay != NULL) {
      if (mPosition < NON_DEFINED_POS)
          pDisplay->drawString("Body position: " + positionText[mPosition]);
      else
          pDisplay->drawString("Body position: " + positionText[NON_DEFINED_POS]);
    };         
  }

  String consumeToJSONstring() {
    //"BODYPOSITION": [ POZITIE ]
    if (isAvailable()) {consume(); return ",\"BODYPOSITION\":[" + String(mPosition) + "]";}
    else return String(""); 
  }

  String consumeToDisplayedString() {
    if (isAvailable()) {consume(); return " BODYPOSITION " + positionText[mPosition] + getEmptyString();}
    else return " BODYPOSITION" + getEmptyString() + getEmptyString();
  }

  uint8_t getBodyPosition(void)
  {
    // Accelerometer acceleration buffer
    byte Version[3];
    // It stores current body position
    uint8_t bodyPos;
  
    // address of the accelerometer
    Wire.beginTransmission(0x0A);
  
    // reset the accelerometer
    Wire.write(0x04); // Y data
    uint8_t error = Wire.endTransmission();
    if (error == 2) // nack on address, missing device
      return NOT_AVAILABLE; // n/a
      
    // request 6 bytes from slave device
    Wire.requestFrom(0x0A, 1);
  
    // slave may send less than requested
    while (Wire.available())
    {
      // receive a byte as character
      Version[0] = Wire.read();
    }
  
    int8_t x_data = (int8_t)Version[0] >> 2;
  
    // address of the accelerometer
    Wire.beginTransmission(0x0A);
  
    // reset the accelerometer
    // Y data
    Wire.write(0x06);
    Wire.endTransmission();
  
    // request 6 bytes from slave device
    Wire.requestFrom(0x0A, 1);
  
    // slave may send less than requested
    while (Wire.available())
    {
      // receive a byte as character
      Version[1] = Wire.read();
    }
  
    int8_t y_data = (int8_t)Version[1] >> 2;
  
    // address of the accelerometer
    Wire.beginTransmission(0x0A);
  
    // reset the accelerometer
    // Y data
    Wire.write(0x08);
    Wire.endTransmission();
  
    // request 6 bytes from slave device
    Wire.requestFrom(0x0A, 1);
  
    // slave may send less than requested
    while (Wire.available())
    {
      // receive a byte as character
      Version[2] = Wire.read();
    }
  
    int8_t z_data = (int8_t)Version[2] >> 2;
  
    delay(100);
  
    // determinate the patient position
    bodyPos = dataToBodyPosition(x_data, y_data, z_data);
  
    #if MYHOSPITAL_DEBUG > 0  
      Serial.println();
      Serial.println("-----------------> BODY POSITION SENSOR");
      // print the X Y Z acceleration raw data
      char bufferAcc[30];
      sprintf (bufferAcc, "Acceleration: X= %d  Y= %d  Z= %d  ", MySignals.x_data, MySignals.y_data, MySignals.z_data);
      Serial.println(bufferAcc);
      // print body position value
      Serial.print("Body Position value = ");
      Serial.println(bodyPos);
    #endif
  
    return bodyPos;
  }
  
  uint8_t dataToBodyPosition(int8_t x_data, int8_t y_data, int8_t z_data)
  {
    uint8_t bodyPos;
    if (z_data >= 12) bodyPos = PRONE_POS;
    else if (y_data >= 12) bodyPos = LEFT_LATERAL_POS;
    else if (y_data <= -12) bodyPos = RIGHT_LATERAL_POS;
    else if (z_data <= -12) bodyPos = SUPINE_LATERAL_POS;
    else if (x_data <= -12) bodyPos = STAND_OR_SIT_POS;
    else bodyPos = NON_DEFINED_POS;

    return bodyPos;
  }
};
