/**
 * ISS Copyright 2019-2020
 * @author Calin Bira (calin.bira@spacescience.ro / calin.bira@upb.ro)
 */
 
#pragma once
class issBodyPositionSensor2 : public issModule {

uint8_t mPosition = 0;
char *MAC_BPS;
issBLE *pBLE;
  
public:
  issBodyPositionSensor2(char *mac) {
    MAC_BPS = mac;
  }
  
  char* getMac() {
    return MAC_BPS;
  }
        
  void init(issDisplay *_pDisplay, issBLE *_pBLE) 
  {
      pBLE = _pBLE;
      pDisplay = _pDisplay;

      if (! pBLE->isInitialized()) pBLE->init(_pDisplay);
      pBLE->pushMAC(MAC_BPS);

      pDisplay = _pDisplay;
      mInitialized = true;
      mPosition = 0;
  }

  String positionText[7] = {
    "n/a", //0
    "Prone position ",  // 1
    "Left lateral   ",  // 2
    "Right lateral  ",  // 3
    "Supine position",  // 4
    "Stand or sit   ",  // 5
    "non-defined    "   // 6
  };

  enum BodyPositions {
    NOT_AVAILABLE       = 0,
    PRONE_POS           = 1,
    LEFT_LATERAL_POS    = 2,
    RIGHT_LATERAL_POS   = 3,
    SUPINE_LATERAL_POS  = 4,
    STAND_OR_SIT_POS    = 5,
    NON_DEFINED_POS     = 6
  };

  String getPositionName(uint8_t val) {
    return positionText[val];
  }

  int getBodyPositionble() {
          #ifndef USE_DUMMY_DATA
          int connected_scale;
          if (pBLE) pBLE->reset();
          int available_scale = MySignals_BLE.scanDevice(MAC_BPS, 1000, TX_POWER_MAX);

          uint16_t y = pDisplay->getNextY();
          if (pDisplay != NULL) pDisplay->drawString("Scale available:", 0, y, pDisplay->getFontSize());
          if (pDisplay != NULL) pDisplay->drawNumber(available_scale, 110, y, pDisplay->getFontSize());

/*
          if (available_scale == 1)
          {
            if (MySignals_BLE.connectDirect(MAC_BPS) == 1)
            {
                if (MySignals_BLE.attributeWrite(MySignals_BLE.connection_handle, SCALE_PROFILE_HANDLE, scale_user_profile, 8) == 0)
                {
                  {
                    if (MySignals_BLE.waitEvent(1000) == BLE_EVENT_ATTCLIENT_ATTRIBUTE_VALUE)
                    {

                      produce();
                      //show();
        }
                  }
                }
            }
          }
          */
         #else
            produce();
         #endif
  }
  
  void loop()
  {
    #ifndef USE_DUMMY_DATA
    int Position = getBodyPositionble(); 
    if ((Position > NOT_AVAILABLE) && (Position < NON_DEFINED_POS)) 
    {
      mPosition = Position;
      produce();
    }
    #else
      mPosition = (mPosition % 6) + 1;
      produce();
    #endif
  }

  void show() {
    if (pDisplay != NULL) {
      if (mPosition < NON_DEFINED_POS)
          pDisplay->drawString("Body position ble: " + positionText[mPosition]);
      else
          pDisplay->drawString("Body position ble: " + positionText[NON_DEFINED_POS]);
    };         
  }

  String consumeToJSONstring() {
    //"BODYPOSITION": [ POZITIE ]
    if (isAvailable()) {consume(); return ",\"BODYPOSITION\":[" + String(mPosition) + "]";}
    else return String("");
  }

  uint8_t getBodyPosition(void)
  {
    // determinate the patient position
    uint8_t bodyPos = dataToBodyPosition(0, 0, 0);
    return bodyPos;
  }
  
  uint8_t dataToBodyPosition(int8_t x_data, int8_t y_data, int8_t z_data)
  {
    uint8_t bodyPos;
    if (z_data >= 12) bodyPos = PRONE_POS;
    else if (y_data >= 12) bodyPos = LEFT_LATERAL_POS;
    else if (y_data <= -12) bodyPos = RIGHT_LATERAL_POS;
    else if (z_data <= -12) bodyPos = SUPINE_LATERAL_POS;
    else if (x_data <= -12) bodyPos = STAND_OR_SIT_POS;
    else bodyPos = NON_DEFINED_POS;

    return bodyPos;
  }
};
