/**
 * ISS Copyright 2020
 * @author Calin Bira (calin.bira@spacescience.ro)
 */
 
#pragma once
#include <Adafruit_GFX_AS.h>
#include <Adafruit_ILI9341_AS.h>

class issGlucometer : issModule {
  
  #define GLUCO_HANDLE1 99
  #define GLUCO_HANDLE2 83

private:
  char *MAC_GLUCO;
  
public:
  struct glucometerBLEDataVector
  {
    uint16_t glucose;
    uint8_t info;  
  };
private:  
  
  glucometerBLEDataVector glucometerBLEData;
  issBLE *pBLE;
  
public:
  issGlucometer(char *mac) {
    MAC_GLUCO = mac;
  }
  
  void init(issDisplay *_pDisplay, issBLE *_pBLE) 
  {
      pBLE = _pBLE;
      pDisplay = _pDisplay;

      if (! pBLE->isInitialized())
        pBLE->init(_pDisplay);
      pBLE->pushMAC(MAC_GLUCO);
      
      mInitialized = true;
      
      glucometerBLEData.glucose = 0;
      glucometerBLEData.info = 0;
  }
public:
  char* getMac() {
    return MAC_GLUCO;
  }
  
  void loop()
  {        
    #ifndef USE_DUMMY_DATA
    uint8_t available_gluco = 0;
    uint8_t connected_gluco = 0;
    uint8_t connection_handle_gluco = 0;
  
    available_gluco = MySignals_BLE.scanDevice(MAC_GLUCO, 1000, TX_POWER_MAX);
  
    //if (pDisplay != NULL) pDisplay->drawString("Gluco available: " + String(available_gluco));
  
    if (available_gluco == 1)
    {  
      if (MySignals_BLE.connectDirect(MAC_GLUCO) == 1)
      {
        if (pDisplay != NULL) pDisplay->drawString("Connected");
        connected_gluco = 1;
  
        uint8_t gluco_subscribe_message[2] = { 0x01 , 0x00 };
        delay(200);
  
        MySignals_BLE.attributeWrite(MySignals_BLE.connection_handle, GLUCO_HANDLE1, gluco_subscribe_message, 2);
        MySignals_BLE.attributeWrite(MySignals_BLE.connection_handle, GLUCO_HANDLE2, gluco_subscribe_message, 2);
        
        delay(200);
        //if (pDisplay != NULL) pDisplay->drawString("Insert blood stripe (20s)");
  
        unsigned long previous = millis();
        do
        {
          if (MySignals_BLE.waitEvent(1000) == BLE_EVENT_ATTCLIENT_ATTRIBUTE_VALUE)
          {
            if (MySignals_BLE.event[8] == 0x0c)
            {
              uint8_t gh = MySignals_BLE.event[12] - 48;
              uint8_t gl = MySignals_BLE.event[13] - 48;
              glucometerBLEData.glucose = (gh * 10) + gl;
              glucometerBLEData.info = 0;

              if (pDisplay != NULL) pDisplay->drawString("Glucose(mg/dl): " + String(glucometerBLEData.glucose));
              produce();
              produce();
            }
            if (MySignals_BLE.event[8] == 0x0d)
            {  
              uint8_t gh = MySignals_BLE.event[12] - 48;
              uint8_t gm = MySignals_BLE.event[13] - 48;
              uint8_t gl = MySignals_BLE.event[14] - 48;
              glucometerBLEData.glucose = (gh * 100) + (gm * 10) + gl;
              glucometerBLEData.info = 0;
  
              if (pDisplay != NULL) pDisplay->drawString("Glucose(mg/dl): " + String(glucometerBLEData.glucose));
              produce();
              produce();
            }
            if (MySignals_BLE.event[8] == 0x0e)
            {
              if (MySignals_BLE.event[12] == 0x4c)
              {
                glucometerBLEData.glucose = 0;
                glucometerBLEData.info = 0xAA;
                //Serial.println(F("LOW GLUCOSE"));
  
                if (pDisplay != NULL) pDisplay->drawString("Glucose(mg/dl): LOW");
              }
              else if (MySignals_BLE.event[12] == 0x48)
              {
                glucometerBLEData.glucose = 360;
                glucometerBLEData.info = 0xBB;
                if (pDisplay != NULL) pDisplay->drawString("Glucose(mg/dl): HIGH");
              }
  
            }
  
            MySignals_BLE.disconnect(MySignals_BLE.connection_handle);
            connected_gluco = 0;
          }
  
        }
        while ((connected_gluco == 1) && ((millis() - previous) < 20000)); //Timeout 20 seconds  
        connected_gluco = 0;
      }
      else
      {
        connected_gluco = 0;
        //if (pDisplay != NULL) pDisplay->drawString("Not Connected");
      }
  
  
    }
    else if (available_gluco == 0)
    {
      //Do nothing
    }
    else
    {
      MySignals_BLE.hardwareReset();
      MySignals_BLE.initialize_BLE_values();
      delay(100);
    }
    delay(1000);
    #else
        glucometerBLEData.glucose = ((glucometerBLEData.glucose + 1) % 1000);
        glucometerBLEData.info = 0;
        produce();
    #endif
  }

//  "GLUCOMETER" : [ CANTITATE_ZAHAR ], // un numar int, 0-999 mg / dl (miligrame pe decilitru)
void show() {
    if (pDisplay != NULL) {
        if (isAvailable())
          pDisplay->drawString("Glucose(mg/dl): " + String(glucometerBLEData.glucose));
        else 
          pDisplay->drawString("Glucose(mg/dl): n/a");
    };         
  }

  String consumeToJSONstring() {
    //"GLUCOMETER": [ CANTITATE_ZAHAR ]
    if (isAvailable()) {consume(); return ",\"GLUCOMETER\":[" + String(glucometerBLEData.glucose) + "]";}
    else return String(""); 
  }

  String consumeToDisplayedString() {
    if (isAvailable()) {consume(); return " GLUCOMETER " + String(glucometerBLEData.glucose) + " mg/dl" + getEmptyString();}
    else return " GLUCOMETER" + getEmptyString();
  }
};
