namespace issMeasurementsNS {
//avoid class & subclass, for performance reasons: use namespace + innner inline static functions

long mCurrentTimestampStart;
long mCurrentTimestampStop;

typedef struct {
  long msStart;
  long msStop;
  const char* message;

#ifdef USE_STATS
  long msSum;
  long msSamples;
#endif

} Measurement_t;

int measurementIndex = 0;
Measurement_t measurements[MAX_MEASUREMENTS];

inline static void start() {
  mCurrentTimestampStart = micros();
}

inline static void stopAndPushnext(char* Message) {
  mCurrentTimestampStop = micros();
  measurements[measurementIndex].message = Message;
  measurements[measurementIndex].msStart = mCurrentTimestampStart;
  measurements[measurementIndex].msStop = mCurrentTimestampStop;

#ifdef USE_STATS
  measurements[measurementIndex].msSum += mCurrentTimestampStop - mCurrentTimestampStart;
  measurements[measurementIndex].msSamples++;
#endif

  measurementIndex = (measurementIndex + 1) % MAX_MEASUREMENTS;
  mCurrentTimestampStart = mCurrentTimestampStop; // prepare for next stop (if no start is issued in-between)
}

#ifdef USE_STATS
inline static void stopAndPush(char* Message) {
  mCurrentTimestampStop = micros();

  for (int i = 0; i < measurementIndex; i++)
    if (measurements[measurementIndex].message == Message) {
      measurements[measurementIndex].msStart = mCurrentTimestampStart;
      measurements[measurementIndex].msStop = mCurrentTimestampStop;
      measurements[measurementIndex].msSum += mCurrentTimestampStop - mCurrentTimestampStart;
      measurements[measurementIndex].msSamples++;
      
      mCurrentTimestampStart = mCurrentTimestampStop;
      return;
    }

  measurements[measurementIndex].message = Message;
  measurements[measurementIndex].msStart = mCurrentTimestampStart;
  measurements[measurementIndex].msStop = mCurrentTimestampStop;

#ifdef USE_STATS
  measurements[measurementIndex].msSum += mCurrentTimestampStop - mCurrentTimestampStart;
  measurements[measurementIndex].msSamples++;
#endif

  measurementIndex = (measurementIndex + 1) % MAX_MEASUREMENTS;
  mCurrentTimestampStart = mCurrentTimestampStop; // prepare for next stop (if no start is issued in-between)
}
#endif

static void pushEmpty() {
  issMeasurementsNS::start();
  issMeasurementsNS::stopAndPushnext("Empty");
}

static void init() {
#ifdef USE_STATS
  for (int i = 0; i < MAX_MEASUREMENTS; i++) {
    measurements[i].msSum = 0;
    measurements[i].msSamples = 0;
  }
#endif
  
  mCurrentTimestampStart = 0;
  issMeasurementsNS::stopAndPush("Init");
}

static inline void erase() {
  measurementIndex = 0;
}


static void pushExample() {
  issMeasurementsNS::init();
  #ifdef USE_STATS
      issMeasurementsNS::stopAndPush("Mesaj");
      issMeasurementsNS::stopAndPush("Mesaj2");
  #endif
  issMeasurementsNS::stopAndPushnext("Mesaj");
}


// compact == true: show no start / stop, just interval and stats
// compact == false: show all data



void print(bool compact = false) {
  SERIALPRINT_MEASUREMENTS.begin(115200);
  if (compact == true)
    for (int i = 0; i < measurementIndex; i++)
      SERIALPRINT_MEASUREMENTS.println(String(measurements[i].message) + ": " + String(measurements[i].msSum / measurements[i].msSamples) + " (" + String(measurements[i].msSamples) + " samples)");
  else {
    for (int i = 0; i < measurementIndex; i++)
      SERIALPRINT_MEASUREMENTS.println(String(measurements[i].message) + ": " + String(measurements[i].msStart) + " us to " + String(measurements[i].msStop) + " us -> " +
                                       String(measurements[i].msSum / measurements[i].msSamples) + " (" + String(measurements[i].msSamples) + " samples)");
  }
}

}
