/**
 * ISS Copyright 2020
 * @author Calin Bira (calin.bira@spacescience.ro / calin.bira@upb.ro)
 */
 
#pragma once
class issModule {  
  protected:
    bool mInitialized = false;
    int mDataItems = 0;
    issDisplay* pDisplay;

  public:
    virtual void init(issDisplay *_pDisplay) {
      pDisplay = _pDisplay;
      mInitialized = true;
    }

    virtual bool isInitialized() {
      return mInitialized;
    }

    virtual void produce() {
      mDataItems++;
    }

    virtual void consume() {
      mDataItems--;
    }

    bool isAvailable() {
      return (mDataItems > 0);
    }
    virtual void show() = 0;
    virtual String consumeToJSONstring() = 0;
    virtual String consumeToDisplayedString() = 0;
    String getEmptyString() {
      return "          ";
    }

  template <class T>
  static T median(T* array, uint8_t length)
  {
    int k, l, m, n;
    T aux[length];
  
    // Initialization of the auxiliar array
    for (k = 0; k < length; k++) aux[k] = 0;
    aux[0] = array[0];
  
    // Ordering of the sensor from lower to higher value
    for (l = 1; l < length; l++)
    {
      for (m = 0; m < l + 1; m++)
      {
        if (array[l] > aux[m])
        {
          for (n = length - 1; n > m; n--) aux[n] = aux[n - 1];
          aux[m] = array[l];
          m = l + 1;
          continue;
        }
      }
    }
  
    // The value in the central position of the array is returned
    return aux[int(length / 2)];
  }
};
