/**
 * ISS Copyright 2019-2020
 * @author Calin Bira (calin.bira@spacescience.ro / calin.bira@upb.ro)
 */
 
#pragma once
class issSPO2 : public issModule {

uint8_t mSaturation = 0;
uint8_t mPulse = 0;
issBLE *pBLE;
char* mMAC;

public:
  issSPO2(char* MAC) {
    mMAC = MAC;
  }

  char* getMac() {
    return mMAC;
  }
  
  void init(issDisplay *_pDisplay, issBLE *_pBLE) 
  {
      pBLE = _pBLE;
      pDisplay = _pDisplay;

      if (! pBLE->isInitialized())
        pBLE->init(_pDisplay);
      pBLE->pushMAC(mMAC);
            
      mInitialized = true;
      mSaturation = 0;
      mPulse = 0;
  }

  void loop()
  {
    #ifndef USE_DUMMY_DATA
      #define SPO2_HANDLE 15
      int spo2;
      int pulse_spo2;
      int connected_spo2;
      int connection_handle_spo2;
  
      if (pBLE) pBLE->reset();
      int available_spo2 = MySignals_BLE.scanDevice(mMAC, 1000, TX_POWER_MAX);
    
      //tft.drawString("SPO2 available:", 0, 30, 2);
      //tft.drawNumber(available_spo2, 110, 30, 2);
  
      if (available_spo2 == 1)
      {  
        if (MySignals_BLE.connectDirect(mMAC) == 1)
        {
          connected_spo2 = 1;
          connection_handle_spo2 = MySignals_BLE.connection_handle;
    
          //tft.drawString("Connected    ", 0, 45, 2);  
          delay(6000);
    
          //To subscribe the spo2 measure write "1" in SPO2_HANDLE
          char attributeData[1] = { 0x01 };
    
          if (MySignals_BLE.attributeWrite(connection_handle_spo2, SPO2_HANDLE, attributeData, 1) == 0)
          {
            //tft.drawString("Subscribed", 0, 60, 2);
    
            unsigned long previous = millis();
            do
            {
              if (MySignals_BLE.waitEvent(1000) == BLE_EVENT_ATTCLIENT_ATTRIBUTE_VALUE)
              {
                char attributeData[1] = {  0x00 };
                MySignals_BLE.attributeWrite(connection_handle_spo2, SPO2_HANDLE, attributeData , 1);
    
                uint8_t pulse_low = MySignals_BLE.event[12];
                pulse_low &= 0b01111111;
    
                uint8_t pulse_high = MySignals_BLE.event[11];
                pulse_high &= 0b01000000;
    
                if (pulse_high == 0)
                  pulse_spo2 = pulse_low;
    
                if (pulse_high == 0b01000000)
                  pulse_spo2 = pulse_low + 0b10000000;
    
                spo2 = MySignals_BLE.event[13];
                spo2 &= 0b01111111;
    
                if ((pulse_spo2 >= 25) && (pulse_spo2 <= 250)
                    && (pulse_spo2 >= 35) && (pulse_spo2 <= 100))
                {
                  mSaturation = spo2;
                  mPulse = pulse_spo2;
                  produce();
                  produce();
                  
                  //sprintf(buffer_tft, "Pulse: %d ppm / SPO2: %d", pulse_spo2, spo2);
                  //tft.drawString(buffer_tft, 0, 75, 2);
    
                  uint16_t errorCode = MySignals_BLE.disconnect(connection_handle_spo2);
                  //tft.drawString("Disconnected", 0, 45, 2);
    
                  connected_spo2 = 0;  
                }
              }
            }
            while ((connected_spo2 == 1) && ((millis() - previous) < 10000));
            connected_spo2 = 0;
          }
          else
          {
            //tft.drawString("Error subscribing", 0, 60, 2);
          }
        }
        else
        {
          connected_spo2 = 0;
          //tft.drawString("Not Connected", 0, 45, 2);
        }
      }
      else if (available_spo2 == 0)
      {
        //Do nothing
      }
      else
      {
        MySignals_BLE.hardwareReset();
        MySignals_BLE.initialize_BLE_values();
        delay(100);
      }
    #else
      mSaturation = ((mSaturation + 1) % 11) + 90;
      mPulse = ((mPulse + 1) % 160) + 50;
      produce();   
      produce(); 
    #endif    
  }

  void show() {
      if (pDisplay != NULL) {
        if (isAvailable())
          pDisplay->drawString("SPO2: " + String(mSaturation) + "%  "  + String(mPulse) + " bpm");
        else 
          pDisplay->drawString("SPO2: n/a");
      }
  }

  //"SPO2" : [PULS_BPM, SATURATIE_IN_PROCENTE]
  String consumeToJSONstring() {
    if (isAvailable()) {consume(); return (",\"SPO2\":[" + String(mPulse) + "," + String(mSaturation)) + "]";}
    else return String(""); 
  }

  String consumeToDisplayedString() {
    if (isAvailable()) {consume(); return " SPO2 " + String(mSaturation) + "%  " + String(mPulse) + " bpm" + getEmptyString();}
    else return " SPO2" + getEmptyString() + getEmptyString();
  }
};
