/**
 * ISS Copyright 2020
 * @author Calin Bira (calin.bira@spacescience.ro / calin.bira@upb.ro)
 */
 //for ESPhacker firmware on ESP8266 module: 
 // see https://github.com/Hieromon/ESPShaker
 
#pragma once
class issWifi5 : issModule {

  Stream *mUart;
  
public:
  void init(issDisplay *disp, Stream *uart) {
    pDisplay = disp;
    mUart = uart;

    //Enable WiFi ESP8266 Power -> bit1:1
    bitSet(MySignals.expanderState, EXP_ESP8266_POWER);
    MySignals.expanderWrite(MySignals.expanderState);
  
    MySignals.initSensorUART();
    MySignals.enableSensorUART(WIFI_ESP8266);

    if (disp) disp->drawString("Waiting for wifi get configured...");
    while (true) {
      String str = Serial.readString();
      if (str.length() > 0)
        if (disp) disp->drawString(str);
      if (str.indexOf("BOOT OK") > 0) break;
    }

    String answer;
    sendCommandToModem("AT+ISSGETIP", "AT+ISSGETIP OK", answer, 5000);
    if (disp) disp->drawString("Done! IP: " + answer);
  }
  
  bool sendCommandToModem(String cmdToSend, String expectedAnswer, 
                            String &answer, int timeoutMs) 
  {
      if (cmdToSend.length() > 0) {
        while(mUart->available() > 0) mUart->read(); // flush rx buffer
        mUart->println(cmdToSend); // send command
      }
      
      if (expectedAnswer.length() == 0) return true;
      long current = millis();

      while (millis() - current < timeoutMs) {
        if (mUart->available() > 0) {
          byte incomingByte = mUart->read();
          answer += (char)incomingByte;
          // if (answer.indexOf(expectedAnswer) >= 0) {
          //   Serial.println("HEY!");
          //   while (mUart->available()) {  
          //     byte incomingByte = mUart->read();
          //     answer += (char)incomingByte;
          //     //Serial.print((char)incomingByte);
          //   }
          //   return true;
          // }
        }
      }
      
      return (answer.indexOf(expectedAnswer) >= 0);
   
  }

  bool waitForAnswer(String expectedAnswer, String &answer, int timeoutMs) {
    long current = millis();
    while (millis() - current < timeoutMs) {
        if (mUart->available() > 0) {
          byte incomingByte = mUart->read();
          answer += (char)incomingByte;
          if (answer.indexOf(expectedAnswer) >= 0) {
            return true;
          }
        }
      }
    return false;
  }

  String buff;
  bool waitForAnswer(int AnswerChars, String &answer, int timeoutMs) {
    int ansLen = 0;
    buff = "";
    long current = millis();
    while (millis() - current < timeoutMs) {
        if (mUart->available() > 0) {
          byte incomingByte = mUart->read();
          if (pDisplay) buff = buff + String((char)incomingByte);
          if (incomingByte == 10)
            if (pDisplay) {pDisplay->drawString(buff); buff = "";}
          answer += (char)incomingByte;
          ansLen++;
          if (ansLen >= AnswerChars) return true;
        }
      }
    return false;
  }

  String lwanswer;
  bool logWifiCommand(String cmd, String expans, int timeout) {
    lwanswer = "";
    String buffer;
    int index, index1;
    
    // //Scale User profile
    // uint8_t user_gender = 1;    //1-Male    0-Female
    // float user_height = 1.32;   //in cm (0-255)
    // uint8_t user_age = 20;      //(0-255)



    bool ret = sendCommandToModem(cmd, expans, lwanswer, timeout);

    //if (pDisplay) pDisplay->drawString(cmd);
    //if (pDisplay) pDisplay->drawString(lwanswer);
    
    
      
        // index = lwanswer.lastIndexOf("genre");
        // index1 = lwanswer.lastIndexOf("}");

        // user_gender = (lwanswer.substring(index + 9, index + 10) == "M") ? 1 : 0;
        // user_height = lwanswer.substring(index + 23, index + 27).toFloat();  
        // user_age = lwanswer.substring(index + 36, index1).toInt();
    
        // Serial.println(user_gender);
        // Serial.println(user_height);
        // Serial.println(user_age);
      

    
    return ret;
  }
  
  bool setup()
  {
    return true;
  }

  String toSend;
  bool sendDataTCP(String SERVER_IP, int SERVER_PORT, String URL, String data) {
    MySignals.initSensorUART();
    MySignals.enableSensorUART(WIFI_ESP8266);

    bool ret = true;
    //toSend = "AT+ISSSENDJSON 86.122.59.60 85 /ehealth/getSensorsData {\"EQUID\":\"00112233445566778899AA\",\"USID\":2,\"BODYPOSITION\":[4]}";
    toSend = "AT+ISSSENDJSON " + SERVER_IP + " " + SERVER_PORT + " " + URL + " " + data;
    
    if (ret) ret = logWifiCommand(toSend, "AT+ISSSENDJSON OK", 10000);
    if (ret) if (pDisplay) Serial.println("Data sent ok");
    if (!ret) if (pDisplay) Serial.println("Data could not be send");
    return ret;
  }

  // String getDataTCP(String SERVER_IP, int SERVER_PORT, String URL, String data) {
  //   MySignals.initSensorUART();
  //   MySignals.enableSensorUART(WIFI_ESP8266);

  //   bool ret = true;
  //   //toSend = "AT+ISSSENDJSON 86.122.59.60 85 /ehealth/getSensorsData {\"EQUID\":\"00112233445566778899AA\",\"USID\":2,\"BODYPOSITION\":[4]}";
  //   toSend = "AT+ISSSENDJSON " + SERVER_IP + " " + SERVER_PORT + " " + URL + " " + data;
    
  //   if (ret) ret = logWifiCommand(toSend, "AT+ISSSENDJSON OK", 10000);
  //   if (ret) if (pDisplay) Serial.println("Data sent ok");
  //   if (!ret) if (pDisplay) Serial.println("Data could not be send");
  //   return ret;
  // }


  void loop() { }
  void show() { }
  String consumeToJSONstring() {return "";}  
  String consumeToDisplayedString() {return "";}
};
