/**
   ISS Copyright 2019-2020
   @author Calin Bira (calin.bira@spacescience.ro / calin.bira@upb.ro)
*/
//https://www.arduino.cc/en/Hacking/PinMapping2560

//---------
// include
//----------
#include <Wire.h>
#include "MySignals.h"

//#define BLE_APP_DEBUG_MESSAGES
//#define USE_DUMMY_DATA

#define MAIN_SERIAL Serial
#define DEBUG_SERIAL Serial
#define TRIGGER_SERIAL Serial3


#include "issDisplay.hpp"
#include "issBattery.hpp"
#include "issTemperature.hpp"
#include "issBodyPositionSensor.hpp"
#include "issSpirometer.hpp"


//BLE drivers follow down:
//#include "issBGlib.hpp"
#include "issBLE.hpp"
#include "issSPO2.hpp"
#include "issBodyScale.hpp"
#include "issBloodPressure.hpp"
#include "issGlucometer.hpp"

// Wifi and TCP driver:
#include "issWifi5.hpp"

#define USE_FILTER_SPEEDTESTS
#ifdef USE_FILTER_SPEEDTESTS
  #include "issFilterSpeedtests.hpp"
#endif

//---------------
// define USE_xx
//---------------
#define USE_BLE
#define USE_WIFI

#define USE_DISPLAY

#define USE_TEMP
#define USE_BATT

#define USE_BODYPOSITION
#define USE_SPIRO

#define USE_SPO2
#define USE_BODYSCALE
#define USE_BLOODPRESSURE
#define USE_GLUCOMETER


//--------------------------
// instances, ifdef USE_xx
//--------------------------
#ifdef USE_DISPLAY
  issDisplay disp;
  issDisplay *pdisp = &disp;
#else
  issDisplay *pdisp = NULL;
#endif

#ifdef USE_BATT
  issBattery battery;
#endif

#ifdef USE_TEMP
  issTemperature temp;
#endif

#ifdef USE_BODYPOSITION
  issBodyPositionSensor bp;
#endif

#ifdef USE_SPIRO
  issSpirometer spiro;
#endif

#ifdef USE_BLE
  issBLE ble;
#endif

#ifdef USE_SPO2
  issSPO2 spo2("00A050041A19");
#endif

#ifdef USE_BODYSCALE
  issBodyScale bodyscale("8CDE5297BF0E");
#endif

#ifdef USE_BLOODPRESSURE
  issBloodPressure bloodpressure("508CB16B1B26");
#endif

#ifdef USE_GLUCOMETER
  issGlucometer glucometer("187A93090CE2");
#endif
//issBodyPositionSensor2 bodypositionble("CD6EBE459D31");

#define SERVER_IP "86.122.59.60"
#define SERVER_PORT 85

//#define SERVER_IP "aces.ddns.net"
//#define SERVER_IP "192.168.1.102"
//#define SERVER_PORT 8888


#ifdef USE_WIFI
  issWifi5 wificard;
#endif

String postData;
void(* resetFunc) (void) = 0;

void setupSAVE() {

    Serial.begin(115200);
    MySignals.begin();
    MySignals.initSensorUART();

    #ifdef USE_DISPLAY
      disp.init(A15, A14);
      disp.setBacklight(HIGH);
      disp.drawString("ISS app started!");
    #endif

    #ifdef USE_WIFI
      wificard.init(pdisp, &MAIN_SERIAL);
      wificard.setup();
    #endif

    #ifdef USE_TEMP
      temp.init(pdisp);
    #endif

    #ifdef USE_BODYPOSITION
      bp.init(pdisp);
    #endif

    #ifdef USE_SPIRO
      spiro.init(pdisp);
    #endif
    TRIGGER_SERIAL.begin(9600);
}

//-----------
// loopSAVE()
//-----------
void loopSAVE() {
//getCurrentTemperature
    #ifdef USE_SPIRO
      spiro.loop();
      spiro.show();
    #endif

    #ifdef USE_TEMP
      temp.loop();
      temp.show();
    #endif

    #ifdef USE_BODYPOSITION
      bp.loop();
      bp.show();
    #endif

    #ifdef USE_BLE
      ble.init(pdisp);
    #endif

    #ifdef USE_SPO2
      spo2.init(pdisp, &ble);
    #endif

    #ifdef USE_BODYSCALE
      bodyscale.init(pdisp, &ble);
    #endif

    #ifdef USE_BLOODPRESSURE
      bloodpressure.init(pdisp, &ble);
    #endif

    #ifdef USE_GLUCOMETER
      glucometer.init(pdisp, &ble);
    #endif

    #ifdef USE_BLE
      ble.enumerate();
    #endif

    #ifdef USE_SPO2
      if (ble.hasfoundMAC(spo2.getMac())) {
          spo2.loop();
          spo2.show();
      }
    #endif

    #ifdef USE_BODYSCALE
      if (ble.hasfoundMAC(bodyscale.getMac())) {
          bodyscale.loop();
          bodyscale.show();
      }
    #endif

    #ifdef USE_BLOODPRESSURE
      if (ble.hasfoundMAC(bloodpressure.getMac())) {
          bloodpressure.loop();
          bloodpressure.show();
      }
    #endif

    #ifdef USE_GLUCOMETER
    if (ble.hasfoundMAC(glucometer.getMac())) {
        glucometer.loop();
        glucometer.show();
    }
    #endif

    #ifdef USE_BLE
      ble.deinit();
    #endif


  /**** DISPLAY ****/
    long startms = millis();
    //disp.drawLogoFast();
    //drawLogoSlow();
    long endms = millis();

    #ifdef USE_DISPLAY
      disp.drawString(String(endms - startms) + " ms to draw logo");
      disp.drawStringForced(String(millis()), disp.getForcedRowX(), 0);
    #endif

    #ifdef USE_TEMP
      pdisp->drawStringForced(         temp.consumeToDisplayedString(), pdisp->getForcedRowX(), pdisp->getForcedRowY(issDisplay::LOGO_SCREEN_TEMPERETURE_ROW));
    #endif

    #ifdef USE_SPIRO
      pdisp->drawStringForced(        spiro.consumeToDisplayedString(), pdisp->getForcedRowX(), pdisp->getForcedRowY(issDisplay::LOGO_SCREEN_SPIROMETER_ROW));
    #endif

    #ifdef USE_SPO2
      pdisp->drawStringForced(         spo2.consumeToDisplayedString(), pdisp->getForcedRowX(), pdisp->getForcedRowY(issDisplay::LOGO_SCREEN_SPO2_ROW));
    #endif

    #ifdef USE_BLOODPRESSURE
      pdisp->drawStringForced(bloodpressure.consumeToDisplayedString(), pdisp->getForcedRowX(), pdisp->getForcedRowY(issDisplay::LOGO_SCREEN_BLOOD_PRESSURE_ROW));
    #endif

    #ifdef USE_GLUCOMETER
      pdisp->drawStringForced(   glucometer.consumeToDisplayedString(), pdisp->getForcedRowX(), pdisp->getForcedRowY(issDisplay::LOGO_SCREEN_GLUCOMETER_ROW));
    #endif

    #ifdef USE_BODYPOSITION
      pdisp->drawStringForced(           bp.consumeToDisplayedString(), pdisp->getForcedRowX(), pdisp->getForcedRowY(issDisplay::LOGO_SCREEN_BODY_POSITION_ROW));
    #endif

    #ifdef USE_BODYSCALE
      pdisp->drawStringForced(    bodyscale.consumeToDisplayedString(), pdisp->getForcedRowX(), pdisp->getForcedRowY(issDisplay::LOGO_SCREEN_BODY_SCALE_ROW));
    #endif
  /**** END_DISPLAY ****/

  /*
    Ca JSON, vom trimite pachete de forma:
    ---- de aici in jos, campuri obligatorii:

    {"EQUID" : "00112233445566778899AA", // 20 caractere hex, 80 biti, 10 Bytes, cu semnificatia equipement ID
    "USID" : int16, //user id, un numar pe 16 biti

    --- de aici in jos orice linie (raport de senzor) este optionala, dar daca vine, vine cu toate coloanele ei (eg SPO2 daca vine,  vine cu 2 campuri mereu):
    "BATTERY": [int16], // in procente 0 - 100
    "SPO2" : [PULS_BPM, SATURATIE_IN_PROCENTE], // PULSE_BPM este int intre 0-999 iar SATURATIE_IN_PROCENTE este int intre 0 si 100
    "TEMP": [T_SUTIMI_DE_GRADE_CELSIUS], // int 0-999 ( echivalent 0 - 99.9 grade Celsius)
    "BLOODPRESSURE" : [SISTOLIC, DIASTOLIC, PULS_BPM], // SISTOLIC este int16 numar intre 0-999 mm Hg, DIASTOLIC 0-999 mm Hg, PULS_BPM 0-999 bpm
    "SPIROMETER" : [ DEBIT, VOLUM ], // DEBIT un numar int 0-999, litri / minut iar VOLUM este un numar int 0-99 litri
    "GLUCOMETER" : [ CANTITATE_ZAHAR ], // un numar int, 0-999 mg / dl (miligrame pe decilitru)
    "BODYPOSITION": [ POZITIE ], // un numar intre 0,1,2,3,4,5 cu semnificatia 0 = N/A,  1 = PE SPATE, 2 = LATERALA STG, 3 = LATERALA DREAPTA, 4 = PE BURTA, 5 = IN PICIOARE sau SEZUT
    "SCALE": [ SUTE_DE_GRAME,  BODY_FAT, MUSCLE_MASS, WATER, CALORIES, BONE_MASS, VISCERAL_FAT ], // SUTE_DE_GRAME un numar int 0-9999 cu sensul de 0-999 kg, BODY_FAT, MUSCLE_MASS, WATER, CALORIES, BONE_MASS, VISCERAL_FAT sunt in procente
    "BUTON": [ STARE ], // STARE = 0 neapasat, 1 = apasat
    }
  */

    const String equipement_idstr = "1234";
    const int user_id = 2;

    postData = "{\"EQUID\":\"" + equipement_idstr + "\",\"USID\":" + String(user_id);
    int postDataSize = postData.length();

    #ifdef USE_BATTERY
      postData = postData + battery.consumeToJSONstring();
    #endif

    #ifdef USE_SPO2
      postData = postData + spo2.consumeToJSONstring();
    #endif

    #ifdef USE_TEMP
      postData = postData + temp.consumeToJSONstring();
    #endif

    #ifdef USE_BLOODPRESSURE
      postData = postData + bloodpressure.consumeToJSONstring();
    #endif

    #ifdef USE_SPIRO
      postData = postData + spiro.consumeToJSONstring();
    #endif

    #ifdef USE_GLUCO
      postData = postData + glucometer.consumeToJSONstring();
    #endif

    #ifdef USE_BODYPOSITION
      postData = postData + bp.consumeToJSONstring();
    #endif

    #ifdef USE_BODYSCALE
      // #ifdef USE_WIFI
      // {
      //   String getPacientData = "123";
      //   int getPacientSize = 0;

      //   while (false == wificard.sendDataTCP(SERVER_IP, SERVER_PORT, "/api/bio-data", getPacientData)) {
      //     pdisp->drawString("Error when getting pacient data, retrying...");
      //     wificard.setup();
          
      //   }
      // }
      // #endif
      postData = postData + bodyscale.consumeToJSONstring();
    #endif    

    int postDataDeltaSize = postData.length() - postDataSize;
    postData = postData + "}";

   
    #ifdef USE_WIFI

      if (postDataDeltaSize > 3) {
        int retries = 1;

        //String SERVER_IP, int SERVER_PORT, String URL, String data
        while (false == wificard.sendDataTCP(SERVER_IP, SERVER_PORT, "/api/save-sensors-data", postData)) {
          pdisp->drawString("Error when sending data, retrying...");
          wificard.setup();
          /*if (++retries == 5) {
            disp.drawString("Cannot send data even after multiple retries. Restarting app...");
            (*resetFunc)();
          }*/
        }
        //pdisp->drawString("Data sent ok in last retry");
        wificard.loop();
      }
    #endif
    }

/****************************************************************************************************************\
 * FUNCTIONS USED FOR TIME AND CURRENT MEASUREMENTS
 ****************************************************************************************************************/

#define MAX_MEASUREMENTS 200
#define SERIALPRINT_MEASUREMENTS Serial
#define USE_STATS
#include "issMeasurements.hpp"
void setupSerial(){
    /*
        sample 5 on stick
        Logs:
            21:35:54.532 -> Init: 0 us to 284 us -> 284 (1 samples)
            21:35:54.532 -> Serial2.begin: 284 us to 316 us -> 32 (1 samples)
            21:35:54.532 -> Serial2.print(0): 316 us to 356 us -> 40 (1 samples)
            21:35:54.532 -> Serial2.print(w): 356 us to 396 us -> 40 (1 samples)
            21:35:54.532 -> Serial2.print(u): 396 us to 448 us -> 52 (1 samples)
            21:35:54.532 -> Serial2.print(u...u): 448 us to 748 us -> 300 (1 samples)

    */
    issMeasurementsNS::init();

    Serial2.begin(115200);
    issMeasurementsNS::stopAndPush("Serial2.begin");

    Serial2.print("0");
    issMeasurementsNS::stopAndPush("Serial2.print(0)");

    Serial2.print("w");
    issMeasurementsNS::stopAndPush("Serial2.print(w)");

    Serial2.print("u");
    issMeasurementsNS::stopAndPush("Serial2.print(u)");

    Serial2.println("uuuuuuuuuuuuuuuuuuuuuuuuuuuuuuu");
    issMeasurementsNS::stopAndPush("Serial2.print(u...u)");

    TRIGGER_SERIAL.begin(115200); TRIGGER_SERIAL.print(0); issMeasurementsNS::print(); while(1);
}

void setupI2C(){
    /* sample 6, 7
        Logs:
            21:45:35.021 -> Init: 0 us to 296 us -> 296 (1 samples)
            21:45:35.021 -> Wire.begin(): 296 us to 336 us -> 40 (1 samples)
            21:45:35.021 -> Wire.beginTransmission(56): 336 us to 364 us -> 28 (1 samples)
            21:45:35.021 -> Wire.write(0): 364 us to 396 us -> 32 (1 samples)
            21:45:35.021 -> Wire.endTransmission(): 396 us to 656 us -> 260 (1 samples)
            21:46:50.445 -> Init: 0 us to 296 us -> 296 (1 samples)
            21:46:50.445 -> Wire.begin(): 296 us to 336 us -> 40 (1 samples)
            21:46:50.445 -> Wire.beginTransmission(56): 336 us to 364 us -> 28 (1 samples)
            21:46:50.445 -> Wire.write(0): 364 us to 396 us -> 32 (1 samples)
            21:46:50.445 -> Wire.endTransmission(): 396 us to 656 us -> 260 (1 samples)


    */
    issMeasurementsNS::init();

    Wire.begin();
    issMeasurementsNS::stopAndPush("Wire.begin()");

    #define EXPANDER    56
    Wire.beginTransmission(EXPANDER);
    issMeasurementsNS::stopAndPush("Wire.beginTransmission(56)");

    Wire.write(0); //disable all power
    issMeasurementsNS::stopAndPush("Wire.write(0)");

    Wire.endTransmission();
    issMeasurementsNS::stopAndPush("Wire.endTransmission()");

    TRIGGER_SERIAL.begin(115200); TRIGGER_SERIAL.print(0); issMeasurementsNS::print(); while(1);
}

void setupIO(){
    /* Samples 8, 9
        Logs:
            21:53:03.732 -> Init: 0 us to 296 us -> 296 (1 samples)
            21:53:03.732 -> digitalWrite(8, LOW): 296 us to 328 us -> 32 (1 samples)
            21:53:03.732 -> digitalRead(8): 328 us to 364 us -> 36 (1 samples)
            21:53:57.263 -> Init: 0 us to 296 us -> 296 (1 samples)
            21:53:57.309 -> digitalWrite(8, LOW): 296 us to 328 us -> 32 (1 samples)
            21:53:57.309 -> digitalRead(8): 328 us to 364 us -> 36 (1 samples)


    */
    issMeasurementsNS::init();

    digitalWrite(8, LOW); // SD_CS = 8
    issMeasurementsNS::stopAndPush("digitalWrite(8, LOW)");

    digitalRead(8);
    issMeasurementsNS::stopAndPush("digitalRead(8)");

    TRIGGER_SERIAL.begin(115200); TRIGGER_SERIAL.print(0); issMeasurementsNS::print(); while(1);
}

void setupSetup(){
    /*
        Samples 10, 11
        Logs:
            21:58:14.071 -> Init: 0 us to 296 us -> 296 (1 samples)
            21:58:14.071 -> M1: 296 us to 324 us -> 28 (1 samples)
            21:58:14.117 -> M123456789: 324 us to 352 us -> 28 (1 samples)
            21:58:14.117 -> M123456789M123456789: 352 us to 380 us -> 28 (1 samples)
            21:59:42.075 -> Init: 0 us to 296 us -> 296 (1 samples)
            21:59:42.075 -> M1: 296 us to 324 us -> 28 (1 samples)
            21:59:42.075 -> M123456789: 324 us to 352 us -> 28 (1 samples)
            21:59:42.075 -> M123456789M123456789: 352 us to 380 us -> 28 (1 samples)
            21:59:55.139 -> Init: 0 us to 296 us -> 296 (1 samples)
            21:59:55.139 -> M1: 296 us to 324 us -> 28 (1 samples)
            21:59:55.139 -> M123456789: 324 us to 352 us -> 28 (1 samples)
            21:59:55.139 -> M123456789M123456789: 352 us to 380 us -> 28 (1 samples)



    */
    issMeasurementsNS::init();

    issMeasurementsNS::stopAndPush("M1");
    issMeasurementsNS::stopAndPush("M123456789");
    issMeasurementsNS::stopAndPush("M123456789M123456789");

    TRIGGER_SERIAL.begin(115200); TRIGGER_SERIAL.print(0); issMeasurementsNS::print(); while(1);
}

void setupBacklight(){
    /*
        Samples 12, 13
        Logs:
            22:07:48.130 -> Init: 0 us to 284 us -> 284 (1 samples)
            22:07:48.130 -> disp.init(): 284 us to 6567184 us -> 6566900 (1 samples)
            22:07:48.130 -> disp.setBacklight(HIGH): 6567184 us to 6567216 us -> 32 (1 samples)
            22:07:48.130 -> disp.drawString(ISS app started!): 6567216 us to 6567292 us -> 76 (1 samples)
            22:07:48.130 -> disp.setBacklight(LOW): 6567292 us to 6567328 us -> 36 (1 samples)
            22:09:27.726 -> Init: 0 us to 284 us -> 284 (1 samples)
            22:09:27.726 -> disp.init(): 284 us to 6567184 us -> 6566900 (1 samples)
            22:09:27.726 -> disp.setBacklight(HIGH): 6567184 us to 6567216 us -> 32 (1 samples)
            22:09:27.726 -> disp.drawString(ISS app started!): 6567216 us to 6567292 us -> 76 (1 samples)
            22:09:27.726 -> disp.setBacklight(LOW): 6567292 us to 6567328 us -> 36 (1 samples)


    */
    issMeasurementsNS::init();

    Serial.begin(115200);
    MySignals.begin();

    #ifdef USE_DISPLAY
      disp.init(A15, A14);
    #endif

    issMeasurementsNS::stopAndPush("disp.init()");

    #ifdef USE_DISPLAY
      disp.setBacklight(HIGH);
    #endif

    issMeasurementsNS::stopAndPush("disp.setBacklight(HIGH)");

    #ifdef USE_DISPLAY
      disp.drawString("ISS app started!");
    #endif

    issMeasurementsNS::stopAndPush("disp.drawString(ISS app started!)");

    #ifdef USE_DISPLAY
      disp.setBacklight(LOW);
    #endif

    issMeasurementsNS::stopAndPush("disp.setBacklight(LOW)");

    TRIGGER_SERIAL.begin(115200); TRIGGER_SERIAL.print(0); issMeasurementsNS::print(); while(1);
}

void setupBitmap(){
    /* Samples 14 and 15
    22:15:46.092 -> Init: 0 us to 284 us -> 284 (1 samples)
    22:15:46.092 -> disp.init(): 284 us to 6567184 us -> 6566900 (1 samples)
    22:15:46.092 -> disp.setBacklight(HIGH): 6567184 us to 6567216 us -> 32 (1 samples)
    22:15:46.139 -> disp.drawString(ISS app started!): 6567216 us to 6567292 us -> 76 (1 samples)
    22:16:50.798 -> Init: 0 us to 284 us -> 284 (1 samples)
    22:16:50.798 -> disp.init(): 284 us to 6567184 us -> 6566900 (1 samples)
    22:16:50.798 -> disp.setBacklight(HIGH): 6567184 us to 6567216 us -> 32 (1 samples)
    22:16:50.798 -> disp.drawString(ISS app started!): 6567216 us to 6567292 us -> 76 (1 samples)
    */

    issMeasurementsNS::init();

    Serial.begin(115200);
    MySignals.begin();

    #ifdef USE_DISPLAY
      disp.init(A15, A14);
    #endif
    issMeasurementsNS::stopAndPush("disp.init()");

    #ifdef USE_DISPLAY
      disp.setBacklight(HIGH);
    #endif

    issMeasurementsNS::stopAndPush("disp.setBacklight(HIGH)");

    #ifdef USE_DISPLAY
      disp.drawString("ISS app started!");
    #endif

    issMeasurementsNS::stopAndPush("disp.drawString(ISS app started!)");

    TRIGGER_SERIAL.begin(115200); TRIGGER_SERIAL.print(0); issMeasurementsNS::print(); while(1);
}

void setupMeasurements(){
    Serial.begin(115200);
    TRIGGER_SERIAL.begin(9600);
    TRIGGER_SERIAL.print("??");
    issMeasurementsNS::init();  TRIGGER_SERIAL.begin(115200); TRIGGER_SERIAL.print(0); issMeasurementsNS::print(); while(1);
    setupSerial();
    setupI2C();
    setupIO();
    setupSetup();

    setupBacklight();
    setupBitmap();
}

#ifdef USE_FILTER_SPEEDTESTS
void setupFilterSpeed() {
    Serial.begin(115200);
    TRIGGER_SERIAL.begin(9600);
    TRIGGER_SERIAL.print("??");

    issMeasurementsNS::init();  TRIGGER_SERIAL.begin(115200); TRIGGER_SERIAL.print(0);

    issFilterSpeedtests fst;
    fst.init();
      fst.runSpeedTest(Filter_t::U8_TESTRUN, 1, 0);
      issMeasurementsNS::stopAndPush("TESTRUN_0");

      fst.runSpeedTest(Filter_t::U8_TESTRUN, 1, 100);
      issMeasurementsNS::stopAndPush("TESTRUN_100");

      fst.runSpeedTest(Filter_t::U8_TESTRUN, 1, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("TESTRUN_512");


      fst.runSpeedTest(Filter_t::U8_MOVING_AVERAGE, 1, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("MAVG1_1000");

      fst.runSpeedTest(Filter_t::U8_MOVING_AVERAGE, 2, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("MAVG2_1000");

      fst.runSpeedTest(Filter_t::U8_MOVING_AVERAGE, 4, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("MAVG4_1000");

      fst.runSpeedTest(Filter_t::U8_MOVING_AVERAGE, 10, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("MAVG10_1000");

      fst.runSpeedTest(Filter_t::U8_MOVING_AVERAGE, 100, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("MAVG100_1000");

      fst.runSpeedTest(Filter_t::U8_MOVING_AVERAGE, 500, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("MAVG500_1000");


      fst.runSpeedTest(Filter_t::U8_MEDIAN, 1, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("MEDIAN1_1000");

      fst.runSpeedTest(Filter_t::U8_MEDIAN, 2, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("MEDIAN2_1000");

      fst.runSpeedTest(Filter_t::U8_MEDIAN, 4, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("MEDIAN4_1000");

      fst.runSpeedTest(Filter_t::U8_MEDIAN, 10, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("MEDIAN10_1000");

      fst.runSpeedTest(Filter_t::U8_MEDIAN, 100, MAX_FILTER_DATA_SIZE);
      issMeasurementsNS::stopAndPush("MEDIAN100_1000");

      //fst.runSpeedTest(Filter_t::U8_MEDIAN, 500, MAX_FILTER_DATA_SIZE);
      //issMeasurementsNS::stopAndPush("MEDIAN500_1000");

      //fst.runSpeedTest(Filter_t::U8_MEDIAN, 1000, MAX_FILTER_DATA_SIZE);
      //issMeasurementsNS::stopAndPush("MEDIAN1000_1024");

    issMeasurementsNS::print();
    while(1);
}
#endif
/****************************************************************************************************************\
 * FUNCTIONS USED TO SWITCH FROM MEASUREMENTS <-> DEFAULT eHEALTH APP
 ****************************************************************************************************************/

void setup() {
  setupSAVE();
  //setupMeasurements();
  //setupFilterSpeed();
}

void loop() {
    #ifdef USE_DISPLAY
    static long old_seconds = millis() / 1024;
    long new_seconds = millis() / 1024;
    if ((new_seconds - old_seconds) > 5*60) {
      disp.drawLogoFast();
      new_seconds = old_seconds;
    }
    #endif

    loopSAVE();
}

//  Rcurrent = 0.925ohm
//  AT+ISSSENDJSON 86.122.59.60 85 /ehealth/getSensorsData {"EQUID":"00112233445566778899AA","USID":2,"BATTERY":[28]}
