import serial
import socket
import time

s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
s.connect(('8.8.8.8', 1))  # connect() for UDP doesn't send packets
local_ip_address = s.getsockname()[0]

def is_slice_in_list(s,l):
    len_s = len(s) #so we don't recompute length of s on every iteration
    return any(s == l[i:len_s+i] for i in range(len(l) - len_s+1))

class SerialWrapper:
    def __init__(self, device):
        self.ser = serial.Serial(device, 115200)

    def sendData(self, data):
        self.ser.write(data.encode())

    def getData(self):
        return self.ser.readline()

def main():
    ser = SerialWrapper('/dev/ttyUSB0')
    #cmdstr = "AT+CJAP=\"ASUS-ACASA\",\"monamona\"";
    #IP = "86.122.59.60"
    #PORT = "85"
    
    IP = local_ip_address;
    PORT = "8888";
    SleepDelay = 0;

    while True:
        data = ser.getData();
        print(data);
        if is_slice_in_list("[WM] freeing allocated params!\r\n".encode(),data):
           break;


    print("milliseconds: answer");
    while True:
        #misc code here
        time.sleep(SleepDelay);
        cmdstr = "AT+ISSCIPSTART TCP " + IP + " " + PORT + "\r\n";
        print(round(time.time()*1000),": ", cmdstr);
        ser.sendData(cmdstr);

        while True:
            data = ser.getData();
            print(data);
            if is_slice_in_list("OK".encode(),data):
               break;

        time.sleep(SleepDelay);

        cmdstr = "AT+ISSCIPSEND 189" + "\r\n";
        print(round(time.time()*1000),": ", cmdstr);
        ser.sendData(cmdstr);

        while True:
            data = ser.getData();
            print(data);
            if is_slice_in_list(">".encode(),data):
               break;

        index = 0;
        time.sleep(1);
        DataPacket = "POST /ehealth/getSensorsData HTTP/1.1";
        #print(DataPacket);
        ser.sendData(DataPacket);
        data = ser.getData();print(round(time.time()*1000),": ", data);

        time.sleep(SleepDelay);
        DataPacket = "Content-Type: application/json";
        #print(DataPacket);
        ser.sendData(DataPacket);
        data = ser.getData();print(round(time.time()*1000),": ", data);

        time.sleep(SleepDelay);
        DataPacket = "Content-Length: 58";
        #print(DataPacket);
        ser.sendData(DataPacket);
        data = ser.getData();print(round(time.time()*1000),": ", data);

        time.sleep(SleepDelay);
        DataPacket = "Host: "+ IP + ":" + PORT;
        #print(DataPacket);
        ser.sendData(DataPacket);
        data = ser.getData();print(round(time.time()*1000),": ", data);

        time.sleep(SleepDelay);
        DataPacket = "Connection: close";
        #print(DataPacket);
        ser.sendData(DataPacket);
        data = ser.getData();print(round(time.time()*1000),": ", data);

        time.sleep(SleepDelay);
        DataPacket = "{\"EQUID\":\"00112233445566778899AA\",\"USID\":2,\"BATTERY\":[28]}";
        #print(DataPacket);
        ser.sendData(DataPacket);

        while True:
            data = ser.getData();
            print(data);
            if is_slice_in_list("Connection: close".encode(),data):
                break;

        time.sleep(SleepDelay);
        cmdstr = "AT+ISSCIPCLOSE\r\n";
        print(round(time.time()*1000),": ", cmdstr);
        ser.sendData(cmdstr);
        data = ser.getData();

        while True:
            data = ser.getData();
            print(data);
            if is_slice_in_list("OK".encode(),data):
                break;

main()
