package gui;
/**
 * GUI
 *
 * @author Calin Bira &lt;calin.bira@upb.ro&gt;
 */

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.concurrent.ConcurrentLinkedQueue;

import com.fazecast.jSerialComm.*;
import threads.PortThread;
import threads.UpdateMessages;
import utils.Utils;

public class GUI extends JFrame implements WindowListener {
    private JPanel panel;
    private JButton startMonitoringButton;
    private JTextArea CRTTextArea;
    private JTextArea MMTextArea;
    private JTextArea WIFITextArea;
    private JButton saveAsButton1;
    private JButton saveAsButton2;
    private JButton saveAsButton3;
    private JLabel statuslabel;
    private JComboBox comboBox1;
    private JRadioButton portOpenRadioButton;

    private UpdateMessages mMessages;
    private Timer mGUItimer;
    private PortThread mThread;
    boolean isPortMonitoringRunning = false;

    public GUI() {
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

        setTitle("MMCRTsplit v0.1");
        setMinimumSize(new Dimension(1536, 800));
        setResizable(false);
        getContentPane().add($$$getRootComponent$$$());

        saveAsButton1.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                JFileChooser fileChooser = new JFileChooser();
                if (fileChooser.showSaveDialog(panel) == JFileChooser.APPROVE_OPTION) {
                    File file = fileChooser.getSelectedFile();
                    saveToFile(file, CRTTextArea.getText());
                }
            }
        });

        saveAsButton2.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                JFileChooser fileChooser = new JFileChooser();
                if (fileChooser.showSaveDialog(panel) == JFileChooser.APPROVE_OPTION) {
                    File file = fileChooser.getSelectedFile();
                    saveToFile(file, MMTextArea.getText());
                }
            }
        });

        saveAsButton3.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                JFileChooser fileChooser = new JFileChooser();
                if (fileChooser.showSaveDialog(panel) == JFileChooser.APPROVE_OPTION) {
                    File file = fileChooser.getSelectedFile();
                    saveToFile(file, WIFITextArea.getText());
                }
            }
        });

        SerialPort[] ports = SerialPort.getCommPorts();
        for (SerialPort port : ports) {
            comboBox1.addItem(port.getSystemPortName());
        }

        startMonitoringButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                if (isPortMonitoringRunning == false) {
                    startPortListening();
                } else {
                    stopPortListening();
                }
            }
        });

        mMessages = new UpdateMessages();
    }

    void startPortListening() {
        isPortMonitoringRunning = true;
        startMonitoringButton.setText("Press to stop monitoring");

        SerialPort[] ports = SerialPort.getCommPorts();
        for (SerialPort port : ports)
            if (comboBox1.getSelectedItem().equals(port.getSystemPortName())) {
                mThread = new PortThread(mMessages, port);
                mThread.start();
            }

        mGUItimer = new Timer(100, new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                updateLabels();
                updatePorts();
            }
        });
        mGUItimer.setRepeats(true);
        mGUItimer.start();
        portOpenRadioButton.setSelected(true);
    }

    void stopPortListening() {
        isPortMonitoringRunning = false;
        if (mThread != null) {
            mThread.interrupt();
            while (mThread.isAlive()) {
                Utils.sleep_ms(10);
            }
        }
        mThread = null;
        mGUItimer.stop();
        startMonitoringButton.setText("Press to start monitoring");
        portOpenRadioButton.setSelected(false);
    }

    void updatePorts() {
        boolean portfound = false;
        SerialPort[] ports = SerialPort.getCommPorts();
        String selectedport = (String) comboBox1.getSelectedItem();
        for (SerialPort port : ports) {
            if (port.getSystemPortName() == selectedport)
                portfound = true;
        }

        if (portfound == false) {
            comboBox1.removeAllItems();
            SerialPort[] newports = SerialPort.getCommPorts();
            for (SerialPort port : newports) {
                comboBox1.addItem(port.getSystemPortName());
                comboBox1.setSelectedIndex(0);
            }
        }
    }

    void updateLabels() {
        if (mMessages == null) return;

        ConcurrentLinkedQueue<String> q;
        q = mMessages.getCRTqueue();
        if (!q.isEmpty()) {
            CRTTextArea.setText(CRTTextArea.getText() + q.remove());
        }

        q = mMessages.getMMqueue();
        if (!q.isEmpty()) {
            WIFITextArea.setText(WIFITextArea.getText() + q.remove());
        }

        q = mMessages.getWIFIqueue();
        if (!q.isEmpty()) {
            WIFITextArea.setText(WIFITextArea.getText() + q.remove());
        }
    }

    void saveToFile(File file, String str) {
        try {
            file.createNewFile();
        } catch (IOException e) {
            statuslabel.setText("Failed to create file");
        }

        if (file.isFile()) {
            FileWriter myWriter = null;
            try {
                myWriter = new FileWriter(file.getAbsolutePath());
                myWriter.write(str);
                //String[] strs = str.split("\\R");
                myWriter.close();
                statuslabel.setText("Saved ok to " + file.getAbsolutePath());
            } catch (IOException e) {
                e.printStackTrace();
                statuslabel.setText("Failed to save");
            }
        } else
            statuslabel.setText("Failed to create file");
    }

    {
// GUI initializer generated by IntelliJ IDEA GUI Designer
// >>> IMPORTANT!! <<<
// DO NOT EDIT OR ADD ANY CODE HERE!
        $$$setupUI$$$();
    }

    /**
     * Method generated by IntelliJ IDEA GUI Designer
     * >>> IMPORTANT!! <<<
     * DO NOT edit this method OR call it in your code!
     *
     * @noinspection ALL
     */
    private void $$$setupUI$$$() {
        panel = new JPanel();
        panel.setLayout(new com.intellij.uiDesigner.core.GridLayoutManager(6, 4, new Insets(0, 0, 0, 0), -1, -1));
        CRTTextArea = new JTextArea();
        CRTTextArea.setText("");
        panel.add(CRTTextArea, new com.intellij.uiDesigner.core.GridConstraints(2, 0, 2, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_CENTER, com.intellij.uiDesigner.core.GridConstraints.FILL_BOTH, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_WANT_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_WANT_GROW, null, new Dimension(150, 50), null, 0, false));
        final com.intellij.uiDesigner.core.Spacer spacer1 = new com.intellij.uiDesigner.core.Spacer();
        panel.add(spacer1, new com.intellij.uiDesigner.core.GridConstraints(2, 1, 2, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_CENTER, com.intellij.uiDesigner.core.GridConstraints.FILL_VERTICAL, 1, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_WANT_GROW, null, null, null, 0, false));
        WIFITextArea = new JTextArea();
        WIFITextArea.setText("");
        panel.add(WIFITextArea, new com.intellij.uiDesigner.core.GridConstraints(2, 3, 2, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_CENTER, com.intellij.uiDesigner.core.GridConstraints.FILL_BOTH, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_WANT_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_WANT_GROW, null, new Dimension(150, 50), null, 0, false));
        saveAsButton1 = new JButton();
        saveAsButton1.setText("Save as ...");
        panel.add(saveAsButton1, new com.intellij.uiDesigner.core.GridConstraints(4, 0, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_CENTER, com.intellij.uiDesigner.core.GridConstraints.FILL_HORIZONTAL, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_SHRINK | com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        saveAsButton2 = new JButton();
        saveAsButton2.setText("Save as ...");
        panel.add(saveAsButton2, new com.intellij.uiDesigner.core.GridConstraints(4, 2, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_CENTER, com.intellij.uiDesigner.core.GridConstraints.FILL_HORIZONTAL, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_SHRINK | com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        saveAsButton3 = new JButton();
        saveAsButton3.setText("Save as ...");
        panel.add(saveAsButton3, new com.intellij.uiDesigner.core.GridConstraints(4, 3, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_CENTER, com.intellij.uiDesigner.core.GridConstraints.FILL_HORIZONTAL, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_SHRINK | com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        MMTextArea = new JTextArea();
        MMTextArea.setText("");
        panel.add(MMTextArea, new com.intellij.uiDesigner.core.GridConstraints(2, 2, 2, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_CENTER, com.intellij.uiDesigner.core.GridConstraints.FILL_BOTH, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_WANT_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_WANT_GROW, null, new Dimension(150, 50), null, 0, false));
        statuslabel = new JLabel();
        statuslabel.setText("Statuslabel");
        panel.add(statuslabel, new com.intellij.uiDesigner.core.GridConstraints(5, 0, 1, 4, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final JLabel label1 = new JLabel();
        label1.setText("CRT");
        panel.add(label1, new com.intellij.uiDesigner.core.GridConstraints(1, 0, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final JLabel label2 = new JLabel();
        label2.setText("MM");
        panel.add(label2, new com.intellij.uiDesigner.core.GridConstraints(1, 2, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final JLabel label3 = new JLabel();
        label3.setText("Wifi");
        panel.add(label3, new com.intellij.uiDesigner.core.GridConstraints(1, 3, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final JPanel panel1 = new JPanel();
        panel1.setLayout(new com.intellij.uiDesigner.core.GridLayoutManager(2, 3, new Insets(0, 0, 0, 0), -1, -1));
        panel.add(panel1, new com.intellij.uiDesigner.core.GridConstraints(0, 0, 1, 4, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_CENTER, com.intellij.uiDesigner.core.GridConstraints.FILL_BOTH, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_SHRINK | com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_SHRINK | com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), null, TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, null, null));
        comboBox1 = new JComboBox();
        panel1.add(comboBox1, new com.intellij.uiDesigner.core.GridConstraints(0, 0, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_HORIZONTAL, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final com.intellij.uiDesigner.core.Spacer spacer2 = new com.intellij.uiDesigner.core.Spacer();
        panel1.add(spacer2, new com.intellij.uiDesigner.core.GridConstraints(1, 0, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_CENTER, com.intellij.uiDesigner.core.GridConstraints.FILL_VERTICAL, 1, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_WANT_GROW, null, null, null, 0, false));
        startMonitoringButton = new JButton();
        startMonitoringButton.setText("Press to start monitoring");
        panel1.add(startMonitoringButton, new com.intellij.uiDesigner.core.GridConstraints(0, 1, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_CENTER, com.intellij.uiDesigner.core.GridConstraints.FILL_HORIZONTAL, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_SHRINK | com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        portOpenRadioButton = new JRadioButton();
        portOpenRadioButton.setText("Port open");
        panel1.add(portOpenRadioButton, new com.intellij.uiDesigner.core.GridConstraints(0, 2, 1, 1, com.intellij.uiDesigner.core.GridConstraints.ANCHOR_WEST, com.intellij.uiDesigner.core.GridConstraints.FILL_NONE, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_SHRINK | com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_CAN_GROW, com.intellij.uiDesigner.core.GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    }

    /**
     * @noinspection ALL
     */
    public JComponent $$$getRootComponent$$$() {
        return panel;
    }

    @Override
    public void windowOpened(WindowEvent windowEvent) {

    }

    @Override
    public void windowClosing(WindowEvent windowEvent) {
        stopPortListening();
    }

    @Override
    public void windowClosed(WindowEvent windowEvent) {
        stopPortListening();
    }

    @Override
    public void windowIconified(WindowEvent windowEvent) {

    }

    @Override
    public void windowDeiconified(WindowEvent windowEvent) {

    }

    @Override
    public void windowActivated(WindowEvent windowEvent) {

    }

    @Override
    public void windowDeactivated(WindowEvent windowEvent) {

    }
}
