package threads;

/**
 * Thread reading serial port from embedded system
 *
 * @author Calin Bira &lt;calin.bira@upb.ro&gt;
 *
 */

import com.fazecast.jSerialComm.SerialPort;
import gui.GUI;

import javax.swing.*;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.InputStreamReader;

public class PortThread extends java.lang.Thread {

    UpdateMessages mUpdates;
    SerialPort mPort;
    volatile boolean mThreadFinished;

    public PortThread(UpdateMessages updates, SerialPort port) {
        mPort = port;
        mThreadFinished = false;
        mUpdates = updates;
    }

    public void interrupt() {
        mThreadFinished = true;
    }

    public void run() {
        BufferedReader reader = null;
        boolean ret = mPort.openPort();
        if (ret == false) mThreadFinished = true;
        else reader = new BufferedReader(new InputStreamReader(mPort.getInputStream()));

        try{
        while (mThreadFinished == false) {
           if (reader.ready()) {
                String line = reader.readLine();
                if (line.startsWith("<CRT>")) {
                    line = line.substring("<CRT>".length());
                    mUpdates.getCRTqueue().add(line + "\r\n");

                } else if (line.startsWith("<MM>")) {
                    line = line.substring("<MM>".length());
                    mUpdates.getMMqueue().add(line + "\r\n");
                } else if (line.startsWith("<WIFI>")) {
                    line = line.substring("<WIFI>".length());
                    mUpdates.getWIFIqueue().add(line + "\r\n");
                }
           }
        }
            if (mPort != null) mPort.closePort();
        }
        catch (Exception ex) {
            mThreadFinished = true;
            if (mPort != null) mPort.closePort();
        }
    }
}
