import asyncio
import platform
import logging

from bleak import discover
from bleak import BleakClient

async def run():
    devices = await discover()
    for d in devices:
        if (d.name == "Moodmetric-42DD"):
             print(d)
             await runexplore(d.address)

async def runexplore(address, debug=True):
    log = logging.getLogger(__name__)
    if debug:
        import sys

        log.setLevel(logging.DEBUG)
        h = logging.StreamHandler(sys.stdout)
        h.setLevel(logging.DEBUG)
        log.addHandler(h)

    async with BleakClient(address) as client:
        log.info(f"Connected: {client.is_connected}")

        for service in client.services:
            log.info(f"[Service] {service}")
            for char in service.characteristics:
                if "read" in char.properties:
                    try:
                        value = bytes(await client.read_gatt_char(char.uuid))
                        valuehex = value.hex()
                        log.info(
                            f"\t[Characteristic] {char} ({','.join(char.properties)}), Value: {valuehex}"
                        )
                        if (char.uuid == "a0956420-9bd2-11e4-bd06-0800200c9a66"):
                            if (value[0] & 1): print("MM not ready")
                            if (value[0] & 2): print("Ring on finger")
                            if (value[0] & 4): print("Battery is out")
                            if (value[0] & 8): print("Any reaction")
                            if (value[0] & 16): print("Strong reaction")
                            if (value[0] & 32): print("MM elevated for 5 minutes")
                            if (value[0] & 64): print("MM low for 5 minutes")
                            print("MM number is " + str(value[1]))
                            print ("Xacc = " + str(value[4]/128 * 2 - 2)+ " g")
                            print ("Yacc = " + str(value[5]/128 * 2 - 2)+ " g")
                            print ("Zacc = " + str(value[6]/128 * 2 - 2)+ " g")

                    except Exception as e:
                        log.error(
                            f"\t[Characteristic] {char} ({','.join(char.properties)}), Value: {e}"
                        )

                else:
                    value = None
                    log.info(
                        f"\t[Characteristic] {char} ({','.join(char.properties)}), Value: {value}"
                    )

                for descriptor in char.descriptors:
                    try:
                        value = bytes(
                            await client.read_gatt_descriptor(descriptor.handle)
                        )
                        log.info(f"\t\t[Descriptor] {descriptor}) | Value: {value}")
                    except Exception as e:
                        log.error(f"\t\t[Descriptor] {descriptor}) | Value: {e}")


loop = asyncio.get_event_loop()
loop.run_until_complete(run())
